#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create CCE Cluster
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    "kind": "Cluster",
    "apiVersion": "v3",
    "metadata": {
        "name": "test2"
    },
    "spec": {
        "type": "VirtualMachine",
        "flavor": "cce.s1.small",
        "version": "v1.13.10-r0",
        "az": "eu-de-01",
        "supportIstio": True,
        "hostNetwork": {
            "vpc": "26ca2783-dc40-4e3a-95b1-5a0756441e12",
            "subnet": "25d24fc8-d019-4a34-9fff-0a09fde6a9cb",
            "SecurityGroup": "f9ae0767-25be-44fc-a21c-5b8a0da66dec"
        },
        "containerNetwork": {
            "mode": "overlay_l2",
            "cidr": "172.16.0.0/16"
        },
        "authentication": {
            "mode": "rbac",
            "authenticatingProxy": {}
        },
        "billingMode": 0,
        "kubernetesSvcIpRange": "10.247.0.0/16",
        "kubeProxyMode": "iptables"
    }
}

conn.cce.create_cluster(**attrs)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.osclient.cce.v2 import cluster
from otcextensions.tests.unit.osclient.cce.v2 import fakes


class TestCluster(fakes.TestCCE):

    def setUp(self):
        super(TestCluster, self).setUp()

    def test_flatten(self):
        _obj = fakes.FakeCluster.create_one()

        flat_data = cluster._flatten_cluster(_obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['endpoint'],
            flat_data['status'],
            flat_data['version'],
            flat_data['type'],
        )

        cmp_data = (
            _obj.metadata.id,
            _obj.metadata.name,
            _obj.status.endpoints['external_otc'],
            _obj.status.status,
            _obj.spec.version,
            _obj.spec.type
        )

        self.assertEqual(data, cmp_data)


class TestClusterList(fakes.TestCCE):

    _objs = fakes.FakeCluster.create_multiple(3)

    columns = ('ID', 'name', 'status', 'flavor', 'version', 'endpoint')

    data = []

    for s in _objs:
        flat_data = cluster._flatten_cluster(s)
        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['status'],
            flat_data['flavor'],
            flat_data['version'],
            flat_data['endpoint'],
        ))

    def setUp(self):
        super(TestClusterList, self).setUp()

        self.cmd = cluster.ListCCECluster(self.app, None)

        self.client.clusters = mock.Mock()

    def test_list_default(self):
        arglist = []

        verifylist = []

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.clusters.side_effect = [
            self._objs
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.clusters.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestClusterShow(fakes.TestCCE):

    _obj = fakes.FakeCluster.create_one()

    columns = ('ID', 'name', 'type', 'status', 'version', 'endpoint',
               'flavor', 'router_id', 'network_id')
    flat_data = cluster._flatten_cluster(_obj)
    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['type'],
        flat_data['status'],
        flat_data['version'],
        flat_data['endpoint'],
        flat_data['flavor'],
        flat_data['router_id'],
        flat_data['network_id'],
    )

    def setUp(self):
        super(TestClusterShow, self).setUp()

        self.cmd = cluster.ShowCCECluster(self.app, None)

        self.client.find_cluster = mock.Mock()

    def test_get(self):
        arglist = [
            'cluster_uuid'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_cluster.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_cluster.assert_called_once_with(
            'cluster_uuid',
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestClusterCreate(fakes.TestCCE):

    _obj = fakes.FakeCluster.create_one()

    columns = ('ID', 'name', 'version', 'endpoint')

    flat_data = cluster._flatten_cluster(_obj)
    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['version'],
        flat_data['endpoint'],
    )

    def setUp(self):
        super(TestClusterCreate, self).setUp()

        self.cmd = cluster.CreateCCECluster(self.app, None)

        self.sdk_client.create_cce_cluster = \
            mock.Mock(return_value=self._obj)

    def test_create(self):
        arglist = [
            'cluster_name',
            'vpc_id',
            'net_id',
            '--description', 'descr',
            '--type', 'VirtualMachine',
            '--flavor', 'flavor',
            '--container-network-mode', 'overlay_l2'
        ]

        verifylist = [
            ('name', 'cluster_name'),
            ('router', 'vpc_id'),
            ('network', 'net_id'),
            ('description', 'descr'),
            ('type', 'VirtualMachine'),
            ('flavor', 'flavor'),
            ('container_network_mode', 'overlay_l2')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.sdk_client.create_cce_cluster.assert_called_once_with(
            container_network_mode='overlay_l2',
            description='descr',
            flavor='flavor',
            name='cluster_name',
            network='net_id',
            router='vpc_id',
            type='VirtualMachine',
            wait=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_wait(self):
        arglist = [
            'cluster_name',
            'vpc_id',
            'net_id',
            '--description', 'descr',
            '--type', 'VirtualMachine',
            '--flavor', 'flavor',
            '--container-network-mode', 'overlay_l2',
            '--wait',
            '--wait-interval', '13'
        ]

        verifylist = [
            ('name', 'cluster_name'),
            ('router', 'vpc_id'),
            ('network', 'net_id'),
            ('description', 'descr'),
            ('type', 'VirtualMachine'),
            ('flavor', 'flavor'),
            ('container_network_mode', 'overlay_l2'),
            ('wait', True),
            ('wait_interval', 13)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.sdk_client.create_cce_cluster.assert_called_once_with(
            container_network_mode='overlay_l2',
            description='descr',
            flavor='flavor',
            name='cluster_name',
            network='net_id',
            router='vpc_id',
            type='VirtualMachine',
            wait=True,
            wait_interval=13
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestClusterDelete(fakes.TestCCE):

    def setUp(self):
        super(TestClusterDelete, self).setUp()

        self.cmd = cluster.DeleteCCECluster(self.app, None)

        self.sdk_client.delete_cce_cluster = mock.Mock()

    def test_delete(self):
        arglist = [
            'cluster_uuid'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.sdk_client.delete_cce_cluster.assert_called_once_with(
            cluster='cluster_uuid', wait=False)

    def test_delete_wait(self):
        arglist = [
            'cluster_uuid',
            '--wait'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid'),
            ('wait', True),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.sdk_client.delete_cce_cluster.assert_called_once_with(
            cluster='cluster_uuid', wait=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.dcs.v1 import instance

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "name": "dcs-a11e",
    "engine": "Redis",
    "capacity": 2,
    "ip": "192.168.3.100",
    "port": 6379,
    "status": "RUNNING",
    "description": "Create a instance",
    "instance_id": FAKE_ID,
    "resource_spec_code": "dcs.single_node",
    "engine_version": "3.0.7",
    "internal_version": None,
    "charging_mode": 0,
    "vpc_id": "27d99e17-42f2-4751-818f-5c8c6c03ff15",
    "vpc_name": "vpc_4944a40e-ac57-4f08-9d38-9786e2759458_192",
    "created_at": "2017-03-31T12:24:46.297Z",
    "error_code": None,
    "product_id": "XXXXXX",
    "security_group_id": "60ea2db8-1a51-4ab6-9e11-65b418c24583",
    "security_group_name": "sg_6379_4944a40e-ac57-4f08-9d38-9786e2759458",
    "subnet_id": "ec2f34b9-20eb-4872-85bd-bea9fc943128",
    "subnet_name": "subnet_az_7f336767-10ec-48a5-9ae8-9cacde119318",
    "available_zones": ["XXXXXX"],
    "max_memory": 460,
    "used_memory": 56,
    "user_id": "6d0977e4c9b74ae7b5a083a8d0d8fafa",
    "user_name": "liutao02",
    "order_id": "XXXXXXXXX",
    "maintain_begin": "22:00:00",
    "maintain_end": "02:00:00"
}


class TestInstance(base.TestCase):

    def setUp(self):
        super(TestInstance, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = instance.Instance()

        self.assertEqual('/instances', sot.base_path)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):

        sot = instance.Instance(**EXAMPLE)
        self.assertEqual(EXAMPLE['instance_id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['engine'], sot.engine)
        self.assertEqual(EXAMPLE['capacity'], sot.capacity)
        self.assertEqual(EXAMPLE['ip'], sot.ip)
        self.assertEqual(EXAMPLE['port'], sot.port)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['resource_spec_code'], sot.resource_spec_code)
        self.assertEqual(EXAMPLE['engine_version'], sot.engine_version)
        self.assertEqual(EXAMPLE['internal_version'], sot.internal_version)
        self.assertEqual(EXAMPLE['charging_mode'], sot.charging_mode)
        self.assertEqual(EXAMPLE['vpc_id'], sot.vpc_id)
        self.assertEqual(EXAMPLE['vpc_name'], sot.vpc_name)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['error_code'], sot.error_code)
        self.assertEqual(EXAMPLE['product_id'], sot.product_id)
        self.assertEqual(EXAMPLE['security_group_id'], sot.security_group_id)
        self.assertEqual(EXAMPLE['security_group_name'],
                         sot.security_group_name)
        self.assertEqual(EXAMPLE['subnet_id'], sot.subnet_id)
        self.assertEqual(EXAMPLE['subnet_name'], sot.subnet_name)
        self.assertEqual(EXAMPLE['available_zones'], sot.available_zones)
        self.assertEqual(EXAMPLE['max_memory'], sot.max_memory)
        self.assertEqual(EXAMPLE['used_memory'], sot.used_memory)
        self.assertEqual(EXAMPLE['user_id'], sot.user_id)
        self.assertEqual(EXAMPLE['user_name'], sot.user_name)
        self.assertEqual(EXAMPLE['maintain_begin'], sot.maintain_begin)
        self.assertEqual(EXAMPLE['maintain_end'], sot.maintain_end)

    def test_stop(self):

        sot = instance.Instance(**EXAMPLE)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {}
        mock_response.headers = {}

        self.sess.put.return_value = mock_response

        sot.stop(self.sess)

        self.sess.put.assert_called_once_with(
            'instances/status',
            json={
                'action': 'stop',
                'instances': [FAKE_ID]}
        )

    def test_start(self):

        sot = instance.Instance(**EXAMPLE)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {}
        mock_response.headers = {}

        self.sess.put.return_value = mock_response

        sot.start(self.sess)

        self.sess.put.assert_called_once_with(
            'instances/status',
            json={
                'action': 'start',
                'instances': [FAKE_ID]}
        )

    def test_restart(self):

        sot = instance.Instance(**EXAMPLE)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {}
        mock_response.headers = {}

        self.sess.put.return_value = mock_response

        sot.restart(self.sess)

        self.sess.put.assert_called_once_with(
            'instances/status',
            json={
                'action': 'restart',
                'instances': [FAKE_ID]}
        )

    def test_change_pwd(self):

        sot = instance.Instance(**EXAMPLE)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {}
        mock_response.headers = {}

        self.sess.put.return_value = mock_response

        sot.change_pwd(self.sess, 'curr', 'new')

        self.sess.put.assert_called_once_with(
            'instances/%s/password' % (FAKE_ID),
            json={
                'old_password': 'curr',
                'new_password': 'new'}
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource
from otcextensions.sdk.modelartsv2.v2.dataset import DatasetVersionSpec


class DatasetVersion(DatasetVersionSpec):
    base_path = "/datasets/%(dataset_id)s/versions"

    resources_key = "versions"

    # capabilities
    allow_create = True
    allow_list = True
    allow_delete = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        "status",
        "train_evaluate_ratio",
        "version_format",
    )

    #: Dataset Version ID.
    version_id = resource.Body("version_id", alternate_id=True)
    #: Dataset ID.
    dataset_id = resource.URI("dataset_id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class ResourceTag(resource.Resource):
    resource_key = 'resource_tag'
    base_path = '/resource-tags'

    _query_mapping = resource.QueryParameters(
        'project_id', 'resource_id', 'resource_type'
    )

    #: Properties
    project_id = resource.Body('project_id')
    tags = resource.Body('tags', type=list)
    resources = resource.Body('resources', type=list)
    resource_id = resource.Body('resource_id')
    resource_type = resource.Body('resource_type')
    key = resource.Body('key')

    #: Allow to create operation for this resource.
    allow_create = True
    #: Allow get operation for this resource.
    allow_fetch = False
    #: Allow update operation for this resource.
    allow_commit = False
    #: Allow to delete operation for this resource.
    allow_delete = True
    #: Allow list operation for this resource.
    allow_list = True
    #: Allow head operation for this resource.
    allow_head = False
    #: Allow patch operation for this resource.
    allow_patch = False

    #: Method for committing a resource (PUT, PATCH, POST)
    commit_method = "PUT"
    #: Method for creating a resource (POST, PUT)
    create_method = "POST"

    requires_id = False

    def create(self, session, prepend_key=False, base_path=None):
        # Overriden here to override prepend_key default value
        self.base_path = "/resource-tags/batch-create"
        return super(ResourceTag, self).create(session, prepend_key, base_path)

    def delete(self, session, prepend_key=False, base_path=None):
        request_body = {
            "project_id": self.project_id,
            "resources": self.resources,
            "tags": self.tags,
        }
        self.base_path = "/resource-tags/batch-delete"
        response = session.post(self.base_path, json=request_body)
        exceptions.raise_from_response(response)
        return response



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv1.v1 import service_event
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes


class TestServiceEvents(fakes.TestModelartsv1):
    _service = fakes.FakeService.create_one()
    objects = fakes.FakeServiceEvent.create_multiple(3)

    column_list_headers = (
        "Occur Time",
        "Event Type",
        "Event Info",
    )

    data = []

    for s in objects:
        data.append(
            (
                cli_utils.UnixTimestampFormatter(s.occur_time),
                s.event_type,
                s.event_info,
            )
        )

    def setUp(self):
        super(TestServiceEvents, self).setUp()

        self.cmd = service_event.ServiceEvents(self.app, None)

        self.client.find_service = mock.Mock(return_value=self._service)
        self.client.service_events = mock.Mock()
        self.client.api_mock = self.client.service_events

    def test_list(self):
        arglist = [self._service.name]

        verifylist = [
            ("service", self._service.name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_service.assert_called_with(
            self._service.name, ignore_missing=False
        )
        self.client.api_mock.assert_called_with(self._service.id)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            self._service.name,
            "--event-type",
            "normal",
            "--limit",
            "10",
            "--offset",
            "2",
            "--order",
            "asc",
            "--sort-by",
            "occur_time",
            "--start-time",
            "123",
            "--end-time",
            "567",
        ]

        verifylist = [
            ("service", self._service.name),
            ("event_type", "normal"),
            ("limit", 10),
            ("offset", 2),
            ("order", "asc"),
            ("sort_by", "occur_time"),
            ("start_time", 123),
            ("end_time", 567),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            self._service.id,
            end_time=567,
            event_type="normal",
            limit=10,
            offset=2,
            order="asc",
            sort_by="occur_time",
            start_time=123,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import message


MESSAGES_EXAMPLE = {
    "messages": [
        {
            "body": "TEST11",
            "attributes": {
                "attribute1": "value1",
                "attribute2": "value2"
            },
        }, {
            "body": {
                "foo": "test02"
            },
            "attributes": {
                "attribute1": "value1",
                "attribute2": "value2"
            },
        }
    ]
}


class TestMessage(base.TestCase):

    def test_basic(self):
        sot = message.Message()

        self.assertEqual(
            '/queues/%(queue_id)s/groups/%(group_id)s/messages',
            sot.base_path)
        self.assertEqual('message', sot.resource_key)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual({
            'ack_wait': 'ack_wait',
            'limit': 'limit',
            'marker': 'marker',
            'max_msgs': 'max_msgs',
            'time_wait': 'time_wait'},
            sot._query_mapping._mapping
        )

    def test_make_it(self):

        sot = message.Message()
        self.assertIsNone(sot.queue_id)
        # TODO()

    def test__collect_attrs(self):
        # TODO()
        pass

    def test_list(self):
        # TODO()
        pass


class TestMessages(base.TestCase):

    def test_basic(self):
        sot = message.Messages()

        self.assertEqual('/queues/%(queue_id)s/messages', sot.base_path)
        self.assertEqual('messages', sot.resources_key)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)

        self.assertDictEqual(
            {
                'limit': 'limit',
                'marker': 'marker',
            },
            sot._query_mapping._mapping
        )

    def test_make_it(self):

        sot = message.Messages(**MESSAGES_EXAMPLE)
        self.assertEqual(2, len(sot.messages))
        # TODO()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestAuth(TestApiG):

    def setUp(self):
        super(TestAuth, self).setUp()

    def test_list_apps_bound_to_api(self):
        found = self.client.list_apps_bound_to_api(
            gateway="560de602c9f74969a05ff01d401a53ed")
        self.assertIsNotNone(list(found))

    def test_list_api_bound_to_app(self):
        attrs = {
            'app_id': '74593f5c94f64a139db38b61a7705df3'
        }
        found = self.client.list_api_bound_to_app(
            gateway="560de602c9f74969a05ff01d401a53ed",
            **attrs)
        self.assertIsNotNone(list(found))

    def test_list_api_not_bound_to_app(self):
        attrs = {
            'app_id': '74593f5c94f64a139db38b61a7705df3',
            'env_id': 'DEFAULT_ENVIRONMENT_RELEASE_ID'
        }
        found = self.client.list_api_not_bound_to_app(
            gateway="560de602c9f74969a05ff01d401a53ed",
            **attrs)
        self.assertIsNotNone(list(found))

    def test_create_auth(self):
        attrs = {
            "env_id": "DEFAULT_ENVIRONMENT_RELEASE_ID",
            "app_ids": ["74593f5c94f64a139db38b61a7705df3"],
            "api_ids": ["64182cc7e77245ebbae8cf3b8522a540"]
        }
        result = self.client.create_auth_in_api(
            gateway="560de602c9f74969a05ff01d401a53ed",
            **attrs)
        self.assertIsNotNone(list(result))

    def test_delete_auth_from_api(self):
        found = list(self.client.list_apps_bound_to_api(
            gateway="560de602c9f74969a05ff01d401a53ed"))
        to_delete = [item for item in found
                     if item['app_id'] == "id"][0]
        self.client.delete_auth_from_api(
            gateway="560de602c9f74969a05ff01d401a53ed",
            auth_id=to_delete.id
        )
        found = list(self.client.list_apps_bound_to_api(
            gateway="560de602c9f74969a05ff01d401a53ed"))
        self.assertEqual(1, len(found))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv1.v1 import model
from otcextensions.tests.unit.sdk.modelartsv1.v1 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.MODEL


class TestModel(base.TestCase):
    def setUp(self):
        super(TestModel, self).setUp()

    def test_basic(self):
        sot = model.Model()

        self.assertEqual("/models", sot.base_path)
        self.assertEqual("models", sot.resources_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_patch)

        self.assertDictEqual(
            {
                "description": "description",
                "limit": "limit",
                "marker": "marker",
                "model_type": "model_type",
                "model_version": "model_version",
                "name": "model_name",
                "not_model_type": "not_model_type",
                "offset": "offset",
                "model_version": "model_version",
                "order": "order",
                "sort_by": "sort_by",
                "status": "model_status",
                "workspace_id": "workspace_id",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        updated_sot_attrs = {
            "create_at": "created_at",
            "model_name": "name",
            "model_id": "id",
            "model_status": "status",
            "model_version": "version",
            "owner": "owner_id",
            "project": "project_id",
            "tenant": "tenant_id",
            "tunable": "is_tunable",
            "market_flag": "is_subscribed",
            "publishable_flag": "is_publishable",
        }
        sot = model.Model(**EXAMPLE)

        for key, value in EXAMPLE.items():
            if key in updated_sot_attrs.keys():
                self.assertEqual(
                    getattr(sot, updated_sot_attrs[key]), EXAMPLE[key]
                )
            else:
                assert_attributes_equal(self, getattr(sot, key), value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.dms.v1 import _base


class AvailabilityZone(_base.Resource):
    """DMS AZ resource"""
    resources_key = 'available_zones'
    base_path = '/availableZones'

    # capabilities
    allow_list = True

    #: Properties
    #: AZ code
    code = resource.Body('code')
    #: Has free resources
    has_available_resources = resource.Body('resource_availability', type=bool)
    #: Port number
    port = resource.Body('port')
    #: Region ID
    region_id = resource.Body('region_id')

    @classmethod
    def list(cls, session, **params):
        return super(AvailabilityZone, cls).list_override(session, **params)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts dataset v2 action implementations"""
import logging

from cliff import columns as cliff_columns
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


VERSION_FORMAT_VALUES_MAP = {
    0: "default format",
    1: "CarbonData (supported only by table datasets)",
    2: "CSV",
}


class RunningTaskType:
    CHOICES_MAP = {
        0: "auto labeling",
        1: "pre-labeling",
        2: "export",
        3: "version switch",
        4: "manifest file export",
        5: "manifest file import",
        6: "version publishing",
        7: "auto grouping",
        10: "one-click model deployment (default value)",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())


class DatasetType(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "image classification",
        1: "object detection",
        100: "text classification",
        101: "named entity recognition",
        102: "text triplet",
        200: "sound classification",
        201: "speech content",
        202: "speech paragraph labeling",
        400: "table dataset",
        600: "video labeling",
        900: "custom format",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


class DatasetStatus(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "creating dataset",
        1: "normal dataset",
        2: "deleting dataset",
        3: "deleted dataset",
        4: "abnormal dataset",
        5: "synchronizing dataset",
        6: "releasing dataset",
        7: "dataset in version switching",
        8: "importing dataset",
    }

    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


_formatters = {
    "labels": cli_utils.YamlFormat,
    "create_time": cli_utils.UnixTimestampFormatter,
    "data_sources": cli_utils.YamlFormat,
    "update_time": cli_utils.UnixTimestampFormatter,
}


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class CreateDataset(command.ShowOne):
    _description = _("Create a modelarts dataset")

    def get_parser(self, prog_name):
        parser = super(CreateDataset, self).get_parser(prog_name)
        parser.add_argument(
            "name",
            metavar="<name>",
            help=_("Dataset name."),
        )
        parser.add_argument(
            "--dataset-type",
            metavar="<dataset_type>",
            choices=list(DatasetType.CHOICES_MAP.keys()),
            type=int,
            required=True,
            help=("Dataset type. Possible values:\n" + DatasetType.STR),
        )
        parser.add_argument(
            "--data-source",
            metavar="data_type=<data_type>,data_path=<data_path>",
            required_keys=["data_type", "data_path"],
            dest="data_sources",
            action=parseractions.MultiKeyValueAction,
            required=True,
            help=_(
                "Input dataset path and type.\n"
                "The following keys are required:\n"
                "data_type=<data_type>: Input data source type\n"
                "data_path=<data_path>: PData source path."
            ),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            help=_("Dataset description."),
        )
        parser.add_argument(
            "--import-annotations",
            action="store_true",
            help=_(
                "Whether to automatically import the labeling information "
                "in the input directory."
            ),
        )
        parser.add_argument(
            "--import-data",
            action="store_true",
            help=_(
                "Whether to import data. This parameter is used only "
                "for table datasets."
            ),
        )
        parser.add_argument(
            "--label-format",
            metavar=(
                "label_type=<label_type>,"
                "text_label_separator=<text_label_separator>,"
                "text_sample_separator=<text_sample_separator>"
            ),
            required_keys=[],
            dest="label_format",
            action=parseractions.MultiKeyValueAction,
            help=_(
                "Label format information. This parameter "
                "is used only for text datasets."
                "label_type=<label_type>: Label type of text classification.\n"
                "text_label_separator=<text_label_separator>: Separator "
                "between labels. By default, a comma (,) is used as the "
                "separator.\n"
                "text_sample_separator=<text_sample_separator>: Separator "
                "between the text and label. By default, the Tab key is "
                "used as the separator."
            ),
        )
        parser.add_argument(
            "--managed",
            action="store_true",
            help=_("Whether to host a dataset."),
        )
        parser.add_argument(
            "--work-path",
            metavar="<work_path>",
            required=True,
            help=_(
                "Output dataset path, which is used to store output files "
                "such as label files. The format is /Bucket name/File path, "
                "for example, /obs-bucket/flower/rose/."
            ),
        )
        parser.add_argument(
            "--work-path-type",
            metavar="<work_path_type>",
            type=int,
            default=0,
            help=_(
                "Type of the dataset output path. "
                "The options are as follows:"
                "\n0: OBS bucket (default value)"
            ),
        )
        parser.add_argument(
            "--workspace-id",
            metavar="<workspace_id>",
            help=_("Workspace ID."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        attrs = {
            "dataset_name": parsed_args.name,
            "work_path_type": parsed_args.work_path_type,
        }
        args_list = (
            "dataset_type",
            "data_sources",
            "description",
            "import_annotations",
            "import_data",
            "label_format",
            "managed",
            "work_path",
            "workspace_id",
        )
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val
        obj = client.create_dataset(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class ListDatasets(command.Lister):
    _description = _("Get List of Modelarts Datasets.")
    columns = (
        "Dataset Id",
        "Dataset Name",
        "Dataset Type",
        "Status",
        "Total Sample Count",
        "Annotated Sample Count",
        "Create Time",
    )

    def get_parser(self, prog_name):
        parser = super(ListDatasets, self).get_parser(prog_name)
        parser.add_argument(
            "--check-running-task",
            action="store_true",
            help=_(
                "Whether to detect tasks (including initialization tasks) "
                "that are running in a dataset."
            ),
        )
        parser.add_argument(
            "--contain-versions",
            action="store_true",
            help=_("Whether the dataset contains a version."),
        )
        parser.add_argument(
            "--dataset-type",
            metavar="<dataset_type>",
            choices=list(DatasetType.CHOICES_MAP.keys()),
            type=int,
            help=("Dataset type. Possible values:\n" + DatasetType.STR),
        )
        parser.add_argument(
            "--file-preview",
            action="store_true",
            help=_("Whether a dataset supports preview when it is queried."),
        )
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Maximum number of records returned on each "
                "page. The default value is 10."
            ),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Start page of the paging list. The default value is 0."),
        )
        parser.add_argument(
            "--order",
            metavar="{asc, desc}",
            type=lambda s: s.lower(),
            choices=["asc", "desc"],
            help=_("Sorting order. Default value: desc"),
        )
        parser.add_argument(
            "--running-task-type",
            metavar="<running_task_type>",
            choices=list(RunningTaskType.CHOICES_MAP.keys()),
            type=int,
            help=_(
                "Type of the running tasks (including initialization tasks) "
                "to be detected. The options are as follows:\n"
                + RunningTaskType.STR
            ),
        )
        parser.add_argument(
            "--search-content",
            metavar="<search_content>",
            help=_("Fuzzy search keyword."),
        )
        parser.add_argument(
            "--sort-by",
            metavar="{create_time, dataset_name}",
            type=lambda s: s.lower(),
            choices=["create_time", "dataset_name"],
            help=_("Sorting field. Default value: publish_at"),
        )
        parser.add_argument(
            "--support-export",
            action="store_true",
            help=_(
                "Whether to filter datasets that can be exported only "
                "(including datasets of image classification, object "
                "detection, and custom format)."
            ),
        )
        parser.add_argument(
            "--train-evaluate-ratio",
            metavar="<train_evaluate_ratio>",
            help=_("Version split ratio for dataset filtering."),
        )
        parser.add_argument(
            "--version-format",
            metavar="<version_format>",
            choices=list(VERSION_FORMAT_VALUES_MAP.keys()),
            type=int,
            help=_(
                "Dataset version format for dataset filtering. This "
                "parameter is used to filter datasets that meet the "
                "filter criteria. The options are as follows:\n"
                + "\n".join(
                    [
                        f"{key}: {value}"
                        for key, value in VERSION_FORMAT_VALUES_MAP.items()
                    ]
                )
            ),
        )
        parser.add_argument(
            "--with-labels",
            action="store_true",
            help=_("Whether to return dataset labels."),
        )
        parser.add_argument(
            "--workspace-id",
            metavar="<workspace_id>",
            help=_("Workspace ID."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        args_list = (
            "check_running_task",
            "contain_versions",
            "dataset_type",
            "file_preview",
            "limit",
            "offset",
            "order",
            "running_task_type",
            "search_content",
            "sort_by",
            "support_export",
            "train_evaluate_ratio",
            "version_format",
            "with_labels",
            "workspace_id",
        )
        query_params = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                query_params[arg] = val

        data = client.datasets(**query_params)

        formatters = {
            "Create Time": cli_utils.UnixTimestampFormatter,
            "Dataset Type": DatasetType,
            "Status": DatasetStatus,
        }

        return (
            self.columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )


class ShowDataset(command.ShowOne):
    _description = _("Show details of a Modelarts Dataset.")

    def get_parser(self, prog_name):
        parser = super(ShowDataset, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        data = client.find_dataset(parsed_args.dataset, ignore_missing=False)

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns, formatters=_formatters)

        return display_columns, data


class UpdateDataset(command.ShowOne):
    _description = _("Modify details of a modelarts Dataset.")

    def get_parser(self, prog_name):
        parser = super(UpdateDataset, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Name or Id of the dataset."),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            help=_("Enter description."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        args_list = ["description"]
        attrs = {}
        for arg in args_list:
            if getattr(parsed_args, arg):
                attrs[arg] = getattr(parsed_args, arg)

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )

        data = client.update_dataset(dataset.id, **attrs)

        display_columns, columns = _get_columns(data)
        data = utils.get_item_properties(data, columns)

        return display_columns, data


class DeleteDataset(command.Command):
    _description = _("Delete Modelarts Dataset(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteDataset, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            nargs="+",
            help=_("Name or Id of the dataset(s) to be deleted."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2
        result = 0
        for name_or_id in parsed_args.dataset:
            try:
                dataset = client.find_dataset(name_or_id, ignore_missing=False)
                client.delete_dataset(dataset.id)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        "Failed to delete dataset with "
                        "name or Id '%(name_or_id)s': %(e)s"
                    ),
                    {"name_or_id": name_or_id, "e": e},
                )
        if result > 0:
            total = len(parsed_args.dataset)
            msg = _(
                "%(result)s of %(total)s Dataset(s) failed " "to delete."
            ) % {"result": result, "total": total}
            raise exceptions.CommandError(msg)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid
import json
import time

import concurrent.futures

from openstackclient.tests.functional import base
from tempest.lib.exceptions import CommandFailed


class TestRdsInstance(base.TestCase):
    """Functional tests for RDS Instance. """

    UUID = uuid.uuid4().hex[:8]
    ROUTER_NAME = 'sdk-test-router-' + UUID
    NET_NAME = 'sdk-test-net-' + UUID
    SUBNET_NAME = 'sdk-test-subnet-' + UUID
    SG_NAME = 'sdk-test-sg-' + UUID
    ROUTER_ID = None
    NET_ID = None
    SG_ID = None

    RDS_NAME = 'sdk-test-rds-' + UUID
    RDS_HA_NAME = 'sdk-test-rds-ha-' + UUID
    RDS_RR_NAME = 'sdk-test-rds-rr-' + UUID
    INSTANCE_LIST = [RDS_NAME, RDS_HA_NAME]

    VOL_TYPE = 'ULTRAHIGH'
    VOL_SIZE = 100
    DATASTORE = 'MySQL'
    VERSION = None

    AZ = 'eu-de-01'
    HA_AZ = 'eu-de-01,eu-de-02'

    FLAVOR = None
    HA_FLAVOR = None
    RR_FLAVOR = None
    REGION = None

    BACKUP_NAME = 'sdk-test-rds-backup-' + UUID
    BACKUP_ID = None

    def test_01_instance_list(self):
        self.openstack(
            'rds instance list -f json '
        )

    def test_02_instance_list_filters(self):
        self.openstack(
            'rds instance list '
            '--limit 1 --id 2 '
            '--name 3 --type Single '
            '--datastore-type PostgreSQL '
            '--router-id 123asd '
            '--network-id 123qwe '
            '--offset 5'
        )

        self.assertTrue(True)

    def test_03_create_instance(self):
        self._initialize_network()
        TestRdsInstance.REGION = self._get_region()
        TestRdsInstance.VERSION = self._get_latest_version()
        TestRdsInstance.FLAVOR = self._get_default_flavor()
        TestRdsInstance.HA_FLAVOR = self._get_default_flavor('ha')
        TestRdsInstance.RR_FLAVOR = self._get_default_flavor('replica')
        with concurrent.futures.ThreadPoolExecutor(max_workers=2) as executor:
            executor.submit(self._create_instance, self.RDS_NAME)
            # Create HA Instance
            executor.submit(
                self._create_instance,
                self.RDS_HA_NAME,
                'semisync')
        for instance in self.INSTANCE_LIST:
            json_output = json.loads(self.openstack(
                'rds instance show -f json ' + instance
            ))
            self.assertEqual(json_output['name'], instance)

    def test_04_create_read_replica(self):
        instance_name = self.INSTANCE_LIST[0]
        json_output = json.loads(self.openstack(
            'rds instance create -f json '
            ' {name} {flavor} '
            ' --replica-of {instance} '
            ' --volume-type {vol_type}'
            ' --size {vol_size} '
            ' --region {region} '
            ' --availability-zone {az} '
            ' --wait --wait-interval 10'.format(
                name=self.RDS_RR_NAME,
                flavor=self.RR_FLAVOR,
                instance=instance_name,
                region=self.REGION,
                vol_type=self.VOL_TYPE,
                vol_size=self.VOL_SIZE,
                az=self.AZ)
        ))
        self.assertTrue(self._wait_for_instance(json_output['id']))
        self.assertIsNotNone(json_output)

    def test_05_backup_list(self):
        instance_name = self.INSTANCE_LIST[0]
        json_output = json.loads(self.openstack(
            'rds backup list -f json ' + instance_name
        ))
        self.assertIsNotNone(json_output)

    def test_06_create_manual_backup(self):
        instance_name = self.INSTANCE_LIST[1]
        json_output = json.loads(self.openstack(
            'rds backup create -f json '
            ' --description sdk-test-backup '
            '{name} {instance} --wait --wait-interval 10'.format(
                name=self.BACKUP_NAME,
                instance=instance_name
            )
        ))
        self.assertIsNotNone(json_output)
        TestRdsInstance.BACKUP_ID = json_output['id']

    def test_07_backup_download_links(self):
        instance_name = self.INSTANCE_LIST[1]
        json_output = json.loads(self.openstack(
            'rds backup list -f json ' + instance_name
        ))
        for backup in json_output:
            json_output = json.loads(self.openstack(
                'rds backup download links -f json ' + backup['ID']
            ))
            self.assertIsNotNone(json_output)

    # def test_08_create_instance_from_backup(self):
    #    json_output = json.loads(self.openstack(
    #        'rds instance show -f json ' + self.INSTANCE_LIST[1]))
    #    from_instance = json_output['id']

    #    restore_instance = self.RDS_NAME + '-restore'
    #    json_output = self._create_instance(
    #        instance_name=restore_instance,
    #        backup=self.BACKUP_ID,
    #        from_instance=from_instance)
    #    self.addCleanup(
    #        self.openstack,
    #        'rds instance delete ' + restore_instance)
    #    self.assertEqual(json_output['status'], 'ACTIVE')

    def test_09_backup_delete(self):
        self.openstack('rds backup delete ' + self.BACKUP_ID)
        self.assertRaises(
            CommandFailed,
            self.openstack,
            'rds backup delete ' + self.BACKUP_ID
        )

    def test_10_backup_policy_set(self):
        instance_name = self.INSTANCE_LIST[0]
        self.openstack(
            'rds instance backup policy set '
            '{instance} --keep-days {keep_days} '
            '--start-time {start_time} '
            '--period {period}'.format(
                instance=instance_name,
                keep_days=5,
                start_time="19:00-20:00",
                period="3,4"
            )
        )

        self.assertTrue(self._wait_for_instance(instance_name))
        json_output = json.loads(self.openstack(
            'rds instance backup policy show -f json ' + instance_name
        ))
        self.assertEqual(json_output['keep_days'], 5)

    def test_11_backup_policy_set_0(self):
        instance_name = self.INSTANCE_LIST[0]
        self.openstack(
            'rds instance backup policy set '
            '{instance} --keep-days {keep_days} '.format(
                instance=instance_name,
                keep_days=0,
            )
        )

        self.assertTrue(self._wait_for_instance(instance_name))
        json_output = json.loads(self.openstack(
            'rds instance backup policy show -f json ' + instance_name
        ))
        self.assertEqual(json_output['keep_days'], 0)

    def test_12_delete_instance(self):
        self.addCleanup(self._deinitialize_network)
        with concurrent.futures.ThreadPoolExecutor(max_workers=2) as executor:
            executor.submit(self._delete_instance, self.RDS_NAME)
            executor.submit(self._delete_instance, self.RDS_HA_NAME)

        for instance in self.INSTANCE_LIST:
            self.assertRaises(
                CommandFailed,
                self.openstack,
                'rds instance show ' + instance
            )
        time.sleep(60)

    def _create_instance(
            self,
            instance_name,
            ha_mode=None,
            backup=None,
            from_instance=None):
        cli_str = ('rds instance create {name}'
                   ' --datastore-type {datastore}'
                   ' --datastore-version {version}'
                   ' --router-id {router_id}'
                   ' --network-id {net_id}'
                   ' --security-group-id {sg_id}'
                   ' --volume-type {vol_type}'
                   ' --size {vol_size}'
                   ' --password Test@123'
                   ' --region {region}'
                   ' --wait --wait-interval 10'
                   ' -f json '.format(
                       name=instance_name,
                       datastore=self.DATASTORE,
                       version=self.VERSION,
                       router_id=self.ROUTER_ID,
                       net_id=self.NET_ID,
                       sg_id=self.SG_ID,
                       vol_type=self.VOL_TYPE,
                       vol_size=self.VOL_SIZE,
                       region=self.REGION))
        if ha_mode:
            cli_str += (' {flavor}'
                        ' --availability-zone {az}'
                        ' --ha-mode {ha_mode}'.format(
                            flavor=self.HA_FLAVOR,
                            az=self.HA_AZ,
                            ha_mode=ha_mode))
        else:
            cli_str += self.FLAVOR + ' --availability-zone ' + self.AZ

        if backup or from_instance:
            cli_str += (' --from-instance {instance}'
                        ' --backup {backup}'.format(
                            instance=from_instance,
                            backup=backup))

        json_output = json.loads(self.openstack(cli_str))

        self._wait_for_instance(json_output['id'])

        return json_output

    def _get_region(self):
        json_output = json.loads(self.openstack(
            'region list -f json'))
        return json_output[0]['Region']

    def _get_latest_version(self):
        json_output = json.loads(self.openstack(
            'rds datastore version list -f json ' + self.DATASTORE))
        versions = [data['Name'] for data in json_output]
        return sorted(versions)[-1]

    def _get_default_flavor(self, flavor_type=None):
        if not self.VERSION:
            TestRdsInstance.VERSION = self._get_latest_datastore_version(
                self.DATASTORE)
        json_output = json.loads(self.openstack(
            'rds flavor list -f json {} {}'.format(
                self.DATASTORE, self.VERSION)
        ))
        if flavor_type:
            flavor_list = [[data['vcpus'], data['name']]
                           for data in json_output if
                           data['instance_mode'] == flavor_type.lower()]
        else:
            flavor_list = [[data['vcpus'], data['name']]
                           for data in json_output if
                           data['instance_mode'] == 'single']
        flavor_list = sorted(flavor_list, key=lambda x: int(x[0]))
        return flavor_list[0][1]

    def _delete_instance(self, instance_name):
        self.openstack(
            'rds instance delete --wait ' + instance_name
        )

    def _initialize_network(self):
        router = json.loads(self.openstack(
            'router create -f json ' + self.ROUTER_NAME
        ))
        net = json.loads(self.openstack(
            'network create -f json ' + self.NET_NAME
        ))
        self.openstack(
            'subnet create {subnet} -f json '
            '--network {net} '
            '--subnet-range 192.168.0.0/24 '.format(
                subnet=self.SUBNET_NAME,
                net=self.NET_NAME
            )
        )
        sg = json.loads(self.openstack(
            'security group create -f json ' + self.SG_NAME
        ))

        self.openstack(
            'router add subnet {router} '
            '{subnet} '.format(
                router=self.ROUTER_NAME,
                subnet=self.SUBNET_NAME
            )
        )

        TestRdsInstance.ROUTER_ID = router['id']
        TestRdsInstance.NET_ID = net['id']
        TestRdsInstance.SG_ID = sg['id']

    def _deinitialize_network(self):
        self.openstack(
            'router remove subnet {router} '
            '{subnet} '.format(
                router=self.ROUTER_NAME,
                subnet=self.SUBNET_NAME
            )
        )
        self.openstack(
            'subnet delete ' + self.SUBNET_NAME
        )
        self.openstack(
            'network delete ' + self.NET_NAME
        )
        self.openstack(
            'router delete ' + self.ROUTER_NAME
        )
        self.openstack(
            'security group delete ' + self.SG_NAME
        )

    def _wait_for_instance(self, instance, timeout=1000, polling=10):
        """
        Wait For Instance Status to return to ACTIVE State
        """
        max_time = time.time() + float(timeout)
        time.sleep(polling)
        while max_time > time.time():
            json_output = json.loads(self.openstack(
                'rds instance show -f json ' + instance
            ))
            status = json_output['status'].lower()
            if status == 'active':
                return True
            else:
                time.sleep(polling)
        return False



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''LoadBalancer v3 action implementations'''
import logging

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_loadbalancer(obj):
    """Flatten the structure of the loadbalancer into a single dict
    """
    data = {
        'id': obj.id,
        'name': obj.name,
        'description': obj.description,
        'provisioning_status': obj.provisioning_status,
        'provider': obj.provider,
        'operating_status': obj.operating_status,
        'ip_address': obj.ip_address,
        'is_admin_state_up': obj.is_admin_state_up,
        'subnet_id': obj.subnet_id,
        'subnet_type': obj.subnet_type,
        'project_id': obj.project_id,
        'port_id': obj.port_id,
        'created_at': obj.created_at,
        'updated_at': obj.updated_at,
        'is_guaranteed': obj.is_guaranteed,
        'vpc_id': obj.vpc_id,
        'availability_zone_list': obj.availability_zones,
        'l4_flavor_id': obj.l4_flavor_id,
        'l7_flavor_id': obj.l7_flavor_id,
        'network_ids': obj.network_ids,
        'deletion_protection_enable': obj.deletion_protection_enable,
        'ip_target_enable': obj.ip_target_enable
    }
    return data


def _add_eips_to_load_balancer_obj(obj, data, columns):
    """Add associated eips to column and data tuples
    """
    i = 0
    for eip in obj.eips:
        data += (eip['eip_id'],)
        columns = columns + ('eip_id_' + str(i + 1),)
        data += (eip['eip_address'],)
        columns = columns + ('eip_address_' + str(i + 1),)
        data += (eip['ip_version'],)
        columns = columns + ('ip_version_' + str(i + 1),)
        i += 1
    return data, columns


def _add_publicips_to_load_balancer_obj(obj, data, columns):
    """Add associated public ips to column and data tuples
    """
    i = 0
    for fip in obj.floating_ips:
        data += (fip['publicip_id'],)
        columns = columns + ('publicip_id_' + str(i + 1),)
        data += (fip['publicip_address'],)
        columns = columns + ('publicip_address_' + str(i + 1),)
        data += (fip['ip_version'],)
        columns = columns + ('publicip_ip_version_' + str(i + 1),)
        i += 1
    return data, columns


def _add_pools_to_load_balancer_obj(obj, data, columns):
    """Add associated pools to column and data tuples
    """
    i = 0
    for pool in obj.pools:
        name = 'pool_id_' + str(i + 1)
        data += (pool['id'],)
        columns = columns + (name,)
        i += 1
    return data, columns


def _add_listeners_to_load_balancer_obj(obj, data, columns):
    """Add associated listeners to column and data tuples
    """
    i = 0
    for s in obj.listeners:
        name = 'listener_id_' + str(i + 1)
        data += (obj.listeners[i]['id'],)
        columns = columns + (name,)
        i += 1
    return data, columns


def _add_tags_to_load_balancer_obj(obj, data, columns):
    data += ('\n'.join((f'value={tag["value"]}, key={tag["key"]}'
                        for tag in obj.tags)),)
    columns = columns + ('tags',)
    return data, columns


def _normalize_tags(tags):
    result = []
    for tag in tags:
        try:
            tag = tag.split('=')
            result.append({
                'key': tag[0],
                'value': tag[1]
            })
        except IndexError:
            result.append({'key': tag[0], 'value': ''})
    return result


class ListLoadBalancers(command.Lister):
    _description = _('List load balancers')
    columns = ('ID',
               'Name',
               'description',
               'provider',
               'operating_status',
               'project_id',
               'vpc_id')

    def get_parser(self, prog_name):
        parser = super(ListLoadBalancers, self).get_parser(prog_name)
        parser.add_argument(
            '--id',
            required=False,
            help=_('Specifies the load balancer ID.')
        )
        parser.add_argument(
            '--name',
            help=_('Specifies the load balancer name.')
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_('Specifies the ID of the last record on the previous page.')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results displayed')
        )
        parser.add_argument(
            '--page-reverse',
            metavar='page_reverse',
            type=bool,
            dest='page_reverse',
            help=_('Specifies the page direction.')
        )
        parser.add_argument(
            '--description',
            help=_('Provides supplementary information about the'
                   'load balancer')
        )
        parser.add_argument(
            '--provisioning-status',
            metavar='<provisioning_status>',
            help=_('Specifies the provisioning status of the load balancer.')
        )
        parser.add_argument(
            '--operating-status',
            metavar='<operating_status>',
            help=_('Specifies the operating status of the load balancer.')
        )
        parser.add_argument(
            '--guaranteed',
            metavar='guaranteed',
            type=bool,
            dest='guaranteed',
            help=_('Specifies whether the load balancer is a dedicated'
                   'load balancer. The value can only be true.')
        )
        parser.add_argument(
            '--vpc-id',
            metavar='<vpc_id>',
            help=_('Specifies the ID of the VPC where the load'
                   'balancer works.')
        )
        parser.add_argument(
            '--port-id',
            dest='vip_port_id',
            help=_('Specifies the ID of the port bound to the virtual'
                   'IP address of the load balancer.')
        )
        parser.add_argument(
            '--ip-address',
            dest='vip_address',
            help=_('Specifies the virtual IP address bound'
                   ' to the load balancer.')
        )
        parser.add_argument(
            '--subnet-id',
            dest='vip_subnet_cidr_id',
            help=_('Specifies the ID of the subnet where the'
                   ' load balancer works.')
        )
        parser.add_argument(
            '--eip',
            dest='eips',
            action='append',
            help=_('Specifies the EIP bound to the load balancer.')
        )
        parser.add_argument(
            '--publicip',
            dest='publicips',
            action='append',
            help=_('Specifies the public IP address bound'
                   'to the load balancer.')
        )
        parser.add_argument(
            '--availability-zone-list',
            metavar='<availability_zone_list>',
            help=_('Specifies the list of AZs where the load balancer'
                   'is created.')
        )
        parser.add_argument(
            '--l4-flavor-id',
            metavar='<l4_flavor_id>',
            help=_('Specifies the ID of the flavor at Layer 4.')
        )
        parser.add_argument(
            '--l7-flavor-id',
            metavar='<l7_flavor_id>',
            help=_('Specifies the ID of the flavor at Layer 7.')
        )
        parser.add_argument(
            '--member-device-id',
            metavar='<member_device_id>',
            help=_('Specifies the ID of the cloud server that serves as'
                   'a backend server.')
        )
        parser.add_argument(
            '--member-address',
            metavar='<member_address>',
            help=_('Specifies the private IP address of the backend server.')
        )
        parser.add_argument(
            '--ip-version',
            metavar='<ip_version>',
            help=_('Specifies the IP version. The value can be 4 (IPv4)'
                   'or 6 (IPv6).')
        )
        parser.add_argument(
            '--subnet-type',
            dest='elb_virsubnet_type',
            help=_('Specifies the type of the subnet on the downstream plane.')
        )
        parser.add_argument(
            '--deletion-protection-enable',
            metavar='<deletion_protection_enable>',
            type=bool,
            help=_('Specifies whether to enable deletion protection.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vlb
        query = {}

        args_list = ['id', 'name', 'marker', 'limit', 'page_reverse',
                     'description', 'provisioning_status', 'operating_status',
                     'guaranteed', 'vpc_id', 'vip_port_id',
                     'eips', 'publicips', 'vip_address', 'vip_subnet_cidr_id',
                     'availability_zone_list', 'l4_flavor_id', 'l7_flavor_id',
                     'member_device_id', 'member_address', 'ip_version',
                     'elb_virsubnet_type', 'deletion_protection_enable']

        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val is not None:
                query[arg] = val
        data = client.load_balancers(**query)
        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_loadbalancer(s), self.columns,
                 ) for s in data))
        return table


class ShowLoadBalancer(command.ShowOne):
    _description = _('Shows details of a Load balancer')
    columns = (
        'id',
        'name',
        'description',
        'subnet_id',
        'subnet_type',
        'port_id',
        'provider',
        'ip_address',
        'provisioning_status',
        'operating_status',
        'project_id',
        'created_at',
        'updated_at',
        'guaranteed',
        'vpc_id',
        'availability_zone_list',
        'l4_flavor_id',
        'l7_flavor_id',
        'network_ids',
        'deletion_protection_enable',
        'ip_target_enable'
    )

    def get_parser(self, prog_name):
        parser = super(ShowLoadBalancer, self).get_parser(prog_name)
        parser.add_argument(
            'loadbalancer',
            metavar='<loadbalancer>',
            help=_('ID or name of the load balancer')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vlb

        obj = client.find_load_balancer(
            name_or_id=parsed_args.loadbalancer,
            ignore_missing=False
        )

        data = utils.get_dict_properties(
            _flatten_loadbalancer(obj), self.columns)
        if obj.eips:
            data, self.columns = _add_eips_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.floating_ips:
            data, self.columns = _add_publicips_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.listeners:
            data, self.columns = _add_listeners_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.pools:
            data, self.columns = _add_pools_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.tags:
            data, self.columns = _add_tags_to_load_balancer_obj(
                obj, data, self.columns)
        return self.columns, data


class CreateLoadBalancer(command.ShowOne):
    _description = _('Create Load balancer')
    columns = (
        'ID',
        'description',
        'provisioning_status',
        'provider',
        'operating_status',
        'ip_address',
        'subnet_id',
        'name',
        'project_id',
        'port_id',
        'created_at',
        'updated_at',
        'guaranteed',
        'vpc_id',
        'availability_zone_list',
        'l4_flavor_id',
        'l7_flavor_id',
        'network_ids'
    )

    def get_parser(self, prog_name):
        parser = super(CreateLoadBalancer, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the load balancer')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('User-defined Load balancer description.')
        )
        parser.add_argument(
            '--ip-address',
            metavar='ip_address',
            dest='vip_address',
            help=_('Specifies the virtual IP address'
                   'bound to the load balancer.')
        )
        parser.add_argument(
            '--subnet-id',
            metavar='subnet_id',
            dest='vip_subnet_cidr_id',
            help=_('Specifies the ID of the IPv4 subnet where the load'
                   'balancer works. ')
        )
        parser.add_argument(
            '--provider',
            metavar='<provider>',
            help=_('Specifies the provider of the load balancer.')
        )
        parser.add_argument(
            '--l4-flavor-id',
            metavar='<l4_flavor_id>',
            help=_('Specifies the ID of the Layer-4 flavor.'
                   'Specify either l4_flavor_id or l7_flavor_id or both'
                   'l4_flavor_id and l7_flavor_id when you create'
                   'a load balancer.')
        )
        parser.add_argument(
            '--l7-flavor-id',
            metavar='<l7_flavor_id>',
            help=_('Specifies the ID of the Layer-7 flavor.'
                   'Specify either l4_flavor_id or l7_flavor_id or both'
                   'l4_flavor_id and l7_flavor_id when you create'
                   'a load balancer.')
        )
        parser.add_argument(
            '--vpc-id',
            metavar='<vpc_id>',
            help=_('Specifies the ID of the VPC where the load balancer'
                   'works.')
        )
        parser.add_argument(
            '--availability-zone',
            dest='availability_zone_list',
            action='append',
            required=True,
            help=_('Specifies the list of AZs where the load balancer'
                   'can be created.')
        )
        parser.add_argument(
            '--tag',
            metavar='key=<keyname1>,value=<value1>',
            action=parseractions.MultiKeyValueAction,
            dest='tags',
            required_keys=['key', 'value'],
            help=_('List of tags. Repeat option for '
                   'multiple tags.\n'
                   'Example:\n'
                   '--tag key=mykey1,value=myvalue1')
        )
        parser.add_argument(
            '--project-id',
            metavar='<project_id>',
            help=_('Specifies the project ID.')
        )
        parser.add_argument(
            '--publicip-id',
            dest='publicip_ids',
            action='append',
            help=_('Specifies the ID of the EIP the system will automatically'
                   'assign and bind to the load balancer during load balancer'
                   'creation. Currently, only the first EIP will be bound to'
                   'the load balancer although multiple EIP IDs can be set')
        )
        parser.add_argument(
            '--ip-version-publicip',
            metavar='<ip_version_publicip>',
            type=int,
            help=_('Specifies the IP address version for public ip.'
                   'Can only be 4 (IPv4). 4 by default.')
        )
        parser.add_argument(
            '--network-type',
            metavar='<network_type>',
            choices=['5_bgp', '5_mailbgp'],
            help=_('Specifies the IP address version for public ip.'
                   'Mandatory for public ip.')
        )
        parser.add_argument(
            '--publicip-description',
            metavar='<publicip_description>',
            help=_('Provides supplementary information about the IPv4 EIP.')
        )
        parser.add_argument(
            '--bandwidth',
            metavar='name=<name>,id=<id>,size=<size>,'
                    'charge_mode=<charge_mode>,share_type=<share_type>,'
                    'billing_info=<billing_info>',
            action=parseractions.MultiKeyValueAction,
            optional_keys=['name', 'size', 'charge_mode', 'billing_info',
                           'share_type', 'billing_info', 'id'],
            help=_('Provides supplementary information about the bandwidth'
                   'for public ip. Mandatory for public ip.')
        )
        parser.add_argument(
            '--network-id',
            metavar='network_id',
            dest='elb_virsubnet_ids',
            action='append',
            help=_('ID of the subnet on the downstream plane.'
                   'The subnets must be in the VPC where the load balancer'
                   'works.'
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--ip-target-enable',
            dest='ip_target_enable',
            action='store_true',
            help=_('Specifies whether to enable IP as a Backend Server.')
        )
        parser.add_argument(
            '--deletion-protection-enable',
            dest='deletion_protection_enable',
            action='store_true',
            help=_('Specifies whether to enable deletion protection for'
                   'the load balancer.')
        )
        return parser

    def take_action(self, parsed_args):
        attrs = {}

        attrs['availability_zone_list'] = \
            parsed_args.availability_zone_list
        attrs['elb_virsubnet_ids'] = parsed_args.elb_virsubnet_ids

        args_list = ['name', 'description', 'vip_address',
                     'vip_subnet_cidr_id', 'provider',
                     'l4_flavor_id', 'l7_flavor_id',
                     'vpc_id', 'availability_zone_list', 'tags',
                     'project_id', 'publicip_ids', 'elb_virsubnet_ids',
                     'ip_target_enable', 'deletion_protection_enable']

        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val is not None:
                attrs[arg] = val

        if parsed_args.ip_target_enable is not None:
            attrs['ip_target_enable'] = parsed_args.ip_target_enable
        if parsed_args.deletion_protection_enable is not None:
            attrs['deletion_protection_enable'] =\
                parsed_args.deletion_protection_enable

        if (parsed_args.network_type or parsed_args.publicip_billing_info
                or parsed_args.ip_version or parsed_args.bandwidth
                or parsed_args.publicip_description):
            attrs['publicip'] = {}
            if parsed_args.bandwidth:
                attrs['publicip']['bandwidth'] = parsed_args.bandwidth[0]
            else:
                attrs['publicip']['bandwidth'] = {}
            if parsed_args.ip_version_publicip:
                attrs['publicip']['ip_version'] =\
                    parsed_args.ip_version_publicip
            if parsed_args.network_type:
                attrs['publicip']['network_type'] = parsed_args.network_type
            if parsed_args.publicip_description:
                attrs['publicip']['description'] \
                    = parsed_args.publicip_description

        client = self.app.client_manager.vlb
        obj = client.create_load_balancer(**attrs)

        data = utils.get_dict_properties(
            _flatten_loadbalancer(obj), self.columns)

        if obj.tags:
            data, self.columns = _add_tags_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.eips:
            data, self.columns = _add_eips_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.floating_ips:
            data, self.columns = _add_publicips_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.listeners:
            data, self.columns = _add_listeners_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.pools:
            data, self.columns = _add_pools_to_load_balancer_obj(
                obj, data, self.columns)
        return self.columns, data


class UpdateLoadBalancer(command.ShowOne):
    _description = _('Update Load balancer')
    columns = (
        'ID',
        'description',
        'provisioning_status',
        'provider',
        'operating_status',
        'ip_address',
        'subnet_id',
        'name',
        'project_id',
        'port_id',
        'created_at',
        'updated_at',
        'guaranteed',
        'vpc_id',
        'availability_zone_list',
        'l4_flavor_id',
        'l7_flavor_id',
        'network_ids'
    )

    def get_parser(self, prog_name):
        parser = super(UpdateLoadBalancer, self).get_parser(prog_name)
        parser.add_argument(
            'loadbalancer',
            metavar='<loadbalancer>',
            help=_('ID or name of the load balancer.')
        )
        parser.add_argument(
            '--name',
            help=_('New name of the load balancer.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Provides supplementary information'
                   'about the load balancer.')
        )
        parser.add_argument(
            'subnet-id',
            metavar='<subnet_id>',
            dest='vip_subnet_cidr_id',
            help=_('Specifies the ID of the IPv4 subnet where'
                   'the load balancer works. The IPv4 subnet can be updated'
                   'using vip_subnet_cidr_id, and the private IPv4 address'
                   'of the load balancer will be changed accordingly.'
                   'If vip_address is also specified, the IP address specified'
                   'by it must be in the subnet specified by'
                   'vip_subnet_cidr_id and will be used as the private'
                   'IPv4 address of the load balancer. The IPv4 subnet'
                   'must be in the VPC'
                   'specified by vpc_id. Enter null if the private IPv4'
                   'address is unbound from the load balancer.')
        )
        parser.add_argument(
            '--ip-address',
            metavar='<ip_address>',
            dest='vip_address',
            help=_('Specifies the virtual IP address'
                   'bound to the load balancer. The IP address must be from'
                   'the IPv4 subnet of the VPC where the load balancer works'
                   'and IP address should not be occupied by other services.'
                   'The IP address specified by this parameter must be in the'
                   'subnet specified by vip_subnet_cidr_id and will be used'
                   'as the private IPv4 address of the load balancer.')
        )
        parser.add_argument(
            '--l4-flavor-id',
            metavar='<l4_flavor_id>',
            help=_('Specifies the ID of the Layer-4 flavor.'
                   'The value cannot be changed from null to a specific value,'
                   'or the other way around. If you need to change the flavor,'
                   'you must select a larger one.')
        )
        parser.add_argument(
            '--l7-flavor-id',
            metavar='<l7_flavor_id>',
            help=_('Specifies the ID of the Layer-7 flavor.'
                   'The value cannot be changed from null to a specific value,'
                   'or the other way around. If you need to change the flavor,'
                   'you must select a larger one.')
        )
        parser.add_argument(
            '--ip-target-enable',
            dest='ip_target_enable',
            action='store_true',
            help=_('Specifies whether to enable IP as a Backend Server.'
                   'The value can only be updated to true.')
        )
        parser.add_argument(
            '--network-id',
            metavar='network_id',
            dest='elb_virsubnet_ids',
            action='append',
            help=_('ID of the subnet on the downstream plane.'
                   'The subnets must be in the VPC where the load balancer'
                   'works.'
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--deletion-protection-enable',
            dest='deletion_protection_enable',
            type=bool,
            help=_('Specifies whether to enable deletion protection for'
                   'the load balancer.')
        )
        return parser

    def take_action(self, parsed_args):
        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.vip_subnet_cidr_id:
            attrs['vip_subnet_cidr_id'] = parsed_args.vip_subnet_cidr_id
        if parsed_args.vip_address:
            attrs['vip_address'] = parsed_args.vip_address
        if parsed_args.l4_flavor_id:
            attrs['l4_flavor_id'] = parsed_args.l4_flavor_id
        if parsed_args.l7_flavor_id:
            attrs['l7_flavor_id'] = parsed_args.l7_flavor_id
        if parsed_args.ip_target_enable:
            attrs['ip_target_enable'] = parsed_args.ip_target_enable
        if parsed_args.elb_virsubnet_ids:
            attrs['elb_virsubnet_ids'] = parsed_args.elb_virsubnet_ids
        if parsed_args.deletion_protection_enable is not None:
            attrs['deletion_protection_enable'] = \
                parsed_args.deletion_protection_enable
        client = self.app.client_manager.vlb
        loadbalancer = client.find_load_balancer(
            name_or_id=parsed_args.loadbalancer,
            ignore_missing=False
        )

        if attrs:
            obj = client.update_load_balancer(
                loadbalancer=loadbalancer.id, **attrs)
        else:
            obj = loadbalancer

        data = utils.get_dict_properties(
            _flatten_loadbalancer(obj), self.columns)

        if obj.tags:
            data, self.columns = _add_tags_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.eips:
            data, self.columns = _add_eips_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.floating_ips:
            data, self.columns = _add_publicips_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.listeners:
            data, self.columns = _add_listeners_to_load_balancer_obj(
                obj, data, self.columns)
        if obj.pools:
            data, self.columns = _add_pools_to_load_balancer_obj(
                obj, data, self.columns)

        return self.columns, data


class DeleteLoadBalancer(command.Command):
    _description = _('Delete load balancer')

    def get_parser(self, prog_name):
        parser = super(DeleteLoadBalancer, self).get_parser(prog_name)
        parser.add_argument(
            'loadbalancer',
            help=_('ID or name of the load balancer')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        loadbalancer = client.find_load_balancer(
            name_or_id=parsed_args.loadbalancer,
            ignore_missing=False
        )

        self.app.client_manager.vlb.delete_load_balancer(
            load_balancer=loadbalancer.id,
            ignore_missing=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.dcaas.v2 import virtual_interface


EXAMPLE = {
    "name": "test-vi",
    "tenant_id": "6fbe9263116a4b68818cf1edce16bc4f",
    "description": "Virtual interface description",
    "direct_connect_id": "456e9263116a4b68818cf1edce16bc4f",
    "vgw_id": "56d5745e-6af2-40e4-945d-fe449be00148",
    "type": "public",
    "service_type": "vpc",
    "vlan": 100,
    "bandwidth": 10,
    "local_gateway_v4_ip": "180.1.1.1/24",
    "remote_gateway_v4_ip": "180.1.1.2/24",
    "route_mode": "static",
    "bgp_asn": 5,
    "bgp_md5": "",
    "remote_ep_group_id": "8ube9263116a4b68818cf1edce16bc4f",
    "service_ep_group_id": "6fbe9263116a4b68818cf1edce16bc4f",
    "create_time": "2016-01-28T16:14:09.466Z",
    "delete_time": "2016-01-28T16:25:27.690Z",
    "admin_state_up": True,
    "rate_limit": False,
    "status": "PENDING_CREATE"
}


class TestVirtualInterface(base.TestCase):

    def setUp(self):
        super(TestVirtualInterface, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = virtual_interface.VirtualInterface()
        self.assertEqual('virtual_interface', sot.resource_key)
        self.assertEqual('virtual_interfaces', sot.resources_key)
        self.assertEqual('/dcaas/virtual-interfaces', sot.base_path)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):

        sot = virtual_interface.VirtualInterface(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['direct_connect_id'], sot.direct_connect_id)
        self.assertEqual(EXAMPLE['vgw_id'], sot.vgw_id)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['service_type'], sot.service_type)
        self.assertEqual(EXAMPLE['vlan'], sot.vlan)
        self.assertEqual(EXAMPLE['bandwidth'], sot.bandwidth)
        self.assertEqual(EXAMPLE['local_gateway_v4_ip'],
                         sot.local_gateway_v4_ip)
        self.assertEqual(EXAMPLE['remote_gateway_v4_ip'],
                         sot.remote_gateway_v4_ip)
        self.assertEqual(EXAMPLE['route_mode'], sot.route_mode)
        self.assertEqual(EXAMPLE['bgp_asn'], sot.bgp_asn)
        self.assertEqual(EXAMPLE['bgp_md5'], sot.bgp_md5)
        self.assertEqual(EXAMPLE['remote_ep_group_id'],
                         sot.remote_ep_group_id)
        self.assertEqual(EXAMPLE['service_ep_group_id'],
                         sot.service_ep_group_id)
        self.assertEqual(EXAMPLE['create_time'], sot.create_time)
        self.assertEqual(EXAMPLE['delete_time'], sot.delete_time)
        self.assertEqual(EXAMPLE['admin_state_up'],
                         sot.admin_state_up)
        self.assertEqual(EXAMPLE['rate_limit'], sot.rate_limit)
        self.assertEqual(EXAMPLE['status'], sot.status)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from otcextensions.sdk.function_graph.v2 import dependency
from otcextensions.tests.functional import base

from openstack import _log

_logger = _log.setup_logging('openstack')


class TestFunctionDependencies(base.BaseFunctionalTest):
    ID = None
    uuid = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestFunctionDependencies, self).setUp()
        u = 'https://fg-test-files.obs.eu-de.otc.t-systems.com/dependency.zip'
        self.attrs = {
            'depend_link': u,
            'depend_type': 'obs',
            'runtime': 'Python3.10',
            'name': 'test-dep-' + self.uuid
        }
        self.dep = self.conn.functiongraph.create_dependency_version(
            **self.attrs
        )
        assert isinstance(self.dep, dependency.Dependency)
        self.addCleanup(
            self.conn.functiongraph.delete_dependency_version,
            self.dep
        )

    def test_list_dependencies(self):
        dep = list(self.conn.functiongraph.dependencies())
        self.assertGreaterEqual(6, len(dep))

    def test_list_dependency_versions(self):
        v = list(self.conn.functiongraph.dependency_versions(self.dep))
        self.assertEqual(1, len(v[0].dependencies))

    def test_get_dependency_version(self):
        v = self.conn.functiongraph.get_dependency_version(self.dep)
        self.assertEqual(v.dep_id, self.dep.dep_id)
        self.assertEqual(v.version, 1)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Adding OBS Dump Task.
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

stream_name = 'test-stream'
attrs = {
    "destination_type": "OBS",
    "obs_destination_descriptor": {
        "task_name": "newtask",
        "consumer_strategy": "LATEST",
        "agency_name": "dis_admin_agency",
        "destination_file_type": "text",
        "obs_bucket_path": "obsbucket",
        "file_prefix": "",
        "partition_format": "yyyy/MM/dd/HH/mm",
        "record_delimiter": "|",
        "deliver_time_interval": 30
    }
}
result = conn.dis.create_dump_task(stream_name, **attrs)
print(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid
import openstack
from openstack import exceptions
from otcextensions.tests.functional.sdk.ctsv3 import TestCtsv3

_logger = openstack._log.setup_logging('openstack')


class TestTracker(TestCtsv3):
    uuid_v4 = uuid.uuid4().hex[:8]
    bucket_name = 'obs-test-' + uuid_v4
    container = None

    def setUp(self):
        super(TestTracker, self).setUp()
        self.client_obs = self.conn.obs
        self.container = self.client_obs.create_container(
            name=self.bucket_name,
            storage_acl='public-read',
        )
        self.cts = self.conn.ctsv3
        attrs = {
            "tracker_type": "system",
            "tracker_name": "system",
            "obs_info": {
                "is_obs_created": False,
                "bucket_name": self.bucket_name,
                "file_prefix_name": "test-prefix"
            },
            "status": "enabled",
            "is_lts_enabled": True,
        }
        self.tracker = self.cts.create_tracker(**attrs)

    def tearDown(self):
        super(TestTracker, self).tearDown()
        try:
            if self.tracker:
                self.client.delete_tracker(self.tracker)
        except exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        objects = self.client_obs.objects(
            container=self.container
        )
        for obj in objects:
            self.client_obs.delete_object(obj, container=self.container)
        self.addCleanup(self.client_obs.delete_container, self.container)

    def test_01_list_trackers(self):
        trackers = list(self.conn.ctsv3.trackers())
        self.assertIsNotNone(trackers)

    def test_02_update_tracker(self):
        attrs = {
            "tracker_type": "system",
            "tracker_name": "system",
            "obs_info": {
                "is_obs_created": False,
                "bucket_name": self.bucket_name,
                "file_prefix_name": "test-prefix"
            },
            "status": "disabled",
            "is_lts_enabled": False,
            "is_support_trace_files_encryption": True,
            "kms_id": "",
            "is_support_validate": False,
        }
        self.conn.ctsv3.update_tracker(**attrs)
        attrs = {
            "tracker_name": "system"
        }
        tracker = list(self.conn.ctsv3.trackers(**attrs))[0]
        self.assertEqual(tracker.status, "disabled")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

import mock
from keystoneauth1 import adapter

from openstack import utils
from openstack.tests.unit import base
from otcextensions.sdk.css.v1 import snapshot

EXAMPLE = {
    'created': '2018-03-07T07:34:47',
    'datastore': {'type': 'elasticsearch', 'version': '7.6.2'},
    'description': '',
    'id': uuid.uuid4().hex,
    'clusterId': uuid.uuid4().hex,
    'clusterName': 'Es-xfx',
    'name': 'snapshot-002',
    'status': 'COMPLETED',
    'updated': '2018-03-07T07:40:12',
    'backupType': '1',
    'backupMethod': 'manual',
    'backupExpectedStartTime': None,
    'backupKeepDay': None,
    'backupPeriod': None,
    'indices': '.kibana,website2',
    'totalShards': 6,
    'failedShards': 0,
    'version': '7.6.2',
    'restoreStatus': 'success',
    'startTime': 1520408087099,
    'endTime': 1520408412219,
    'bucketName': 'obs-b8ed',
}

EXAMPLE_POLICY = {
    'keepday': 2,
    'period': '16:00 GMT+08:00',
    'prefix': 'snapshot',
    'bucket': 'es-backup',
    'basePath': 'css_repository/tests',
    'agency': 'usearch',
    'enable': 'true',
    'indeices': '*',
    'snapshotCmkId': uuid.uuid4().hex,
    'frequency': 'DAY',
}


class TestSnapshot(base.TestCase):
    def setUp(self):
        super(TestSnapshot, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = snapshot.Snapshot()

        self.assertEqual(
            '/clusters/%(uri_cluster_id)s/index_snapshot', sot.base_path
        )
        self.assertEqual('backup', sot.resource_key)
        self.assertEqual('backups', sot.resources_key)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        sot = snapshot.Snapshot(**EXAMPLE)
        updated_sot_attrs = (
            'clusterId',
            'clusterName',
            'backupType',
            'backupMethod',
            'backupPeriod',
            'backupExpectedStartTime',
            'backupKeepDay',
            'totalShards',
            'failedShards',
            'startTime',
            'endTime',
            'bucketName',
            'restoreStatus',
            'created',
            'updated',
        )
        self.assertEqual(EXAMPLE['clusterId'], sot.cluster_id)
        self.assertEqual(EXAMPLE['clusterName'], sot.cluster_name)
        self.assertEqual(EXAMPLE['backupType'], sot.backup_type)
        self.assertEqual(EXAMPLE['backupMethod'], sot.backup_method)
        self.assertEqual(EXAMPLE['backupPeriod'], sot.backup_period)
        self.assertEqual(
            EXAMPLE['backupExpectedStartTime'], sot.backup_start_time
        )
        self.assertEqual(EXAMPLE['backupKeepDay'], sot.backup_keep_days)
        self.assertEqual(EXAMPLE['created'], sot.created_at)
        self.assertEqual(EXAMPLE['updated'], sot.updated_at)
        self.assertEqual(EXAMPLE['totalShards'], sot.total_shards)
        self.assertEqual(EXAMPLE['failedShards'], sot.failed_shards)
        self.assertEqual(EXAMPLE['startTime'], sot.start_time)
        self.assertEqual(EXAMPLE['endTime'], sot.end_time)
        self.assertEqual(EXAMPLE['bucketName'], sot.bucket_name)
        self.assertEqual(EXAMPLE['restoreStatus'], sot.restore_status)
        for key, value in EXAMPLE.items():
            if key not in updated_sot_attrs:
                self.assertEqual(getattr(sot, key), value)

    def test_restore(self):
        sot = snapshot.Snapshot.existing(id=EXAMPLE['id'])
        cluster_id = uuid.uuid4().hex
        json_body = {
            'targetCluster': uuid.uuid4().hex,
            'indices': 'myindex1,myindex2',
        }
        response = mock.Mock()
        response.status_code = 201
        response.headers = {}
        self.sess.post.return_value = response

        rt = sot.restore(self.sess, cluster_id, **json_body)
        uri = utils.urljoin(
            'clusters', cluster_id, 'index_snapshot', sot.id, 'restore'
        )
        self.sess.post.assert_called_with(uri, json=json_body)

        self.assertIsNone(rt)


class TestSnapshotPolicy(base.TestCase):
    def setUp(self):
        super(TestSnapshotPolicy, self).setUp()

    def test_basic(self):
        sot = snapshot.SnapshotPolicy()

        self.assertEqual(
            '/clusters/%(cluster_id)s/index_snapshot/policy', sot.base_path
        )
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        sot = snapshot.SnapshotPolicy(**EXAMPLE_POLICY)
        self.assertEqual(EXAMPLE_POLICY['agency'], sot.agency)
        self.assertEqual(EXAMPLE_POLICY['basePath'], sot.backup_path)
        self.assertEqual(EXAMPLE_POLICY['frequency'], sot.frequency)
        self.assertEqual(EXAMPLE_POLICY['period'], sot.backup_period)
        self.assertEqual(EXAMPLE_POLICY['keepday'], sot.backup_keep_days)
        self.assertEqual(EXAMPLE_POLICY['bucket'], sot.bucket_name)
        self.assertEqual(EXAMPLE_POLICY['snapshotCmkId'], sot.cmk_id)
        self.assertEqual(EXAMPLE_POLICY['prefix'], sot.prefix)
        self.assertEqual(bool(EXAMPLE_POLICY['enable']), sot.is_enabled)


class TestSnapshotConfiguration(base.TestCase):
    def setUp(self):
        super(TestSnapshotConfiguration, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = snapshot.SnapshotConfiguration()

        self.assertEqual(
            '/clusters/%(cluster_id)s/index_snapshot/%(setting)s',
            sot.base_path,
        )
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_disable(self):
        sot = snapshot.SnapshotConfiguration.existing(cluster_id=EXAMPLE['id'])
        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.delete.return_value = response

        rt = sot.disable(self.sess)
        uri = utils.urljoin('clusters', sot.cluster_id, 'index_snapshots')
        self.sess.delete.assert_called_with(uri)

        self.assertIsNone(rt)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import trigger

EXAMPLE = {
    "trigger_id": "9a14fae1-78cf-4185-ac7a-429eb6dc41fb",
    "trigger_type_code": "TIMER",
    "trigger_status": "ACTIVE",
    "event_data": {
        "name": "Timer-cpg3",
        "schedule": "3m",
        "schedule_type": "Rate"
    },
    "updated_at": "2022-11-09 16:37:24",
    "created_at": "2022-11-09 16:37:24"
}


class TestFunctionTrigger(base.TestCase):

    def test_basic(self):
        sot = trigger.Trigger()
        path = '/fgs/triggers/%(function_urn)s'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = trigger.Trigger(**EXAMPLE)
        self.assertEqual(EXAMPLE['trigger_id'], sot.trigger_id)
        self.assertEqual(EXAMPLE['trigger_type_code'], sot.trigger_type_code)
        self.assertEqual(EXAMPLE['trigger_status'], sot.trigger_status)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE['updated_at'], sot.created_at)
        self.assertEqual(EXAMPLE['event_data']['name'], sot.event_data.name)
        self.assertEqual(
            EXAMPLE['event_data']['schedule'],
            sot.event_data.schedule
        )
        self.assertEqual(
            EXAMPLE['event_data']['schedule_type'],
            sot.event_data.schedule_type
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time
from urllib.parse import urlsplit

from openstack import resource
from openstack import _log
from openstack import exceptions
from openstack import proxy
from otcextensions.sdk.dws.v1 import cluster as _cluster
from otcextensions.sdk.dws.v1 import flavor as _flavor
from otcextensions.sdk.dws.v1 import snapshot as _snapshot
from otcextensions.sdk.dws.v1 import tag as _tag

LOG = _log.setup_logging(__name__)


def _format_cluster_response(obj):
    if hasattr(obj, 'endpoints'):
        private_domain = []
        for endpoint in obj.endpoints:
            private_domain.append(endpoint['connect_info'])
        setattr(obj, 'private_domain', private_domain)
    if hasattr(obj, 'public_endpoints'):
        public_domain = []
        for public_endpoint in obj.public_endpoints:
            public_domain.append(public_endpoint['public_connect_info'])
        setattr(obj, 'public_domain', public_domain)
    return obj


class Proxy(proxy.Proxy):
    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {
            'Accept': 'application/json',
            'Content-type': 'application/json',
        }

    # ======== Cluster ========
    def clusters(self, **query):
        """List all Clusters.

        :returns: a generator of
            (:class:`~otcextensions.sdk.dws.v1.cluster.Cluster`) instances
        """
        if query.get('limit'):
            query.update(paginated=False)
        return self._list(_cluster.Cluster, **query)

    def get_cluster(self, cluster):
        """Get the cluster by UUID

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`

        :returns: instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        """
        obj = self._get(_cluster.Cluster, cluster)
        return _format_cluster_response(obj)

    def find_cluster(self, name_or_id, ignore_missing=True):
        """Find a single cluster

        :param name_or_id: The name or ID of a DWS cluster
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the cluster does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent cluster.

        :returns:
            One :class:`~otcextensions.sdk.dws.v1.cluster.Cluster` or ``None``
        """
        obj = self._find(
            _cluster.Cluster, name_or_id, ignore_missing=ignore_missing
        )
        return _format_cluster_response(obj)

    def create_cluster(self, **attrs):
        """Create a cluster from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`,
            comprised of the properties on the Cluster class.
        :returns: The results of cluster creation
        :rtype: :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        """
        return self._create(_cluster.Cluster, **attrs)

    def restart_cluster(self, cluster):
        """Get the cluster by UUID

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`

        :returns: instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.restart(self)

    def scale_out_cluster(self, cluster, node_count):
        """Scaling Out a Cluster Nodes

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        :param count: Number of nodes to be scaled out.

        :returns: instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.scale_out(self, node_count)

    def reset_password(self, cluster, new_password):
        """Reset the password of cluster administrator.

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        :param new_password: New password of the GaussDB(DWS)
            cluster administrator.

        :returns: instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.reset_password(self, new_password)

    def delete_cluster(
        self, cluster, keep_last_manual_snapshot=0, ignore_missing=True
    ):
        """Delete a DWS Cluster

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        :param int keep_last_manual_snapshot: The number of latest manual
            snapshots that need to be retained for a cluster.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the cluster does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent cluster.
        :returns: None
        """
        return self._delete(
            _cluster.Cluster,
            cluster,
            keep_last_manual_snapshot=keep_last_manual_snapshot,
            ignore_missing=ignore_missing,
        )

    # ======== Flavors ========
    def flavors(self):
        """List all Flavors.

        :returns: a generator of
            (:class:`~otcextensions.sdk.dws.v1.flavor.Flavor`) instances
        """

        split_url = urlsplit(self.get_endpoint())
        url = '{}://{}/v2/{}/node-types'.format(
            split_url.scheme, split_url.netloc, self.get_project_id()
        )
        return self._list(_flavor.Flavor, base_path=url)

    # ======== Snapshot ========
    def snapshots(self):
        """List all Snapshots of a cluster.

        :returns: a generator of
            (:class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot`) instances
        """
        return self._list(_snapshot.Snapshot)

    def get_snapshot(self, snapshot):
        """Get a Snapshot details

        :param snapshot: key id or an instance of
            :class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot`

        :returns: instance of
            :class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot`
        """
        return self._get(_snapshot.Snapshot, snapshot)

    def find_snapshot(self, name_or_id, ignore_missing=True):
        """Find a single snapshot

        :param name_or_id: The name or ID of a snapshot
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the snapshot does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent snapshot.

        :returns:
            One :class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot` or
            ``None``
        """
        return self._find(
            _snapshot.Snapshot, name_or_id, ignore_missing=ignore_missing
        )

    def create_snapshot(self, **attrs):
        """Create a cluster Snapshot from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot`,
            comprised of the properties on the Snapshot class.
        :returns: The results of cluster snapshot creation
        :rtype: :class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot`
        """
        return self._create(
            _snapshot.Snapshot,
            **attrs,
        )

    def delete_snapshot(self, snapshot, ignore_missing=True):
        """Delete a snapshot

        :param snapshot: key id or an instance of
            :class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the snapshot does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent snapshot.
        :returns: ``None``
        """
        return self._delete(
            _snapshot.Snapshot,
            snapshot,
            ignore_missing=ignore_missing,
        )

    def restore_snapshot(self, snapshot, **attrs):
        """Restore a snapshot.

        :param snapshot: key id or an instance of
            :class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot`
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dws.v1.snapshot.Restore`,
            comprised of the properties on the Restore class.
        :returns: instance of
            :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
        """
        snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
        obj = self._create(_snapshot.Restore, snapshot_id=snapshot.id, **attrs)
        if hasattr(obj, 'cluster'):
            return obj.cluster
        return obj

    def wait_for_cluster(self, cluster, interval=5, wait=1800):
        """Wait for a Cluster status to be `AVAILABLE`.

        :param cluster: The value can be the ID of a cluster
            or a :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
            instance.
        :param int interval:
            Number of seconds to wait before two consecutive checks.
            Default is 5.
        :param int wait:
            Maximum number of seconds to wait for the status change.
            Default is 1800.
        :return: ``True`` on success.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
            transitions to a failure status.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
            transition to the desired status does not occur within
            the specified time.
        """
        end_time = time.time() + wait

        task_status_list = (
            'CONFIGURING_EXT_DATASOURCE',
            'DELETING_EXT_DATASOURCE',
            'GROWING',
            'REBOOTING',
            'REDISTRIBUTING',
            'RESTORING',
            'SETTING_CONFIGURATION',
            'SNAPSHOTTING',
        )

        while time.time() < end_time:
            obj = self._get(_cluster.Cluster, cluster)
            status = obj.status
            task_status = obj.task_status
            action_progress = obj.action_progress
            sub_status = obj.sub_status

            if status == 'CREATING':
                LOG.debug(
                    'Still waiting for resource %s to reach state %s, '
                    'current state is %s',
                    obj.name,
                    'AVAILABLE',
                    status,
                )
                time.sleep(interval)
            elif task_status in task_status_list or action_progress:
                LOG.debug(
                    'Still waiting for resource %s task_status to complete, '
                    'current task_status is %s',
                    obj.name,
                    task_status,
                )
                time.sleep(interval)
            elif sub_status == 'NORMAL' and status == 'AVAILABLE':
                return True
            else:
                raise exceptions.ResourceFailure(
                    f'Failed! Cluster status: {status} '
                    f'task_status: {task_status} '
                    f'sub_status: {sub_status} '
                    f'action_progress: {action_progress}'
                )
        raise exceptions.ResourceTimeout(
            'Wait Timed Out. Cluster action still in progress.'
        )

    def wait_for_cluster_scale_out(self, cluster, interval=5, wait=1800):
        """Wait for a Cluster Scale Out Task to Complete.

        :param cluster: The value can be the ID of a cluster
            or a :class:`~otcextensions.sdk.dws.v1.cluster.Cluster`
            instance.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 5.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 1800
        """
        obj = self._get(_cluster.Cluster, cluster)
        is_snapshotting = obj.task_status == 'SNAPSHOTTING'
        self.wait_for_cluster(cluster, interval, wait)
        if is_snapshotting:
            time.sleep(60)
            self.wait_for_cluster(cluster, interval, wait)

    def cluster_tags(self, cluster):
        """
        List tags for a DWS cluster.

        :param cluster: Cluster ID or an instance of
            `otcextensions.sdk.dws.v1.cluster.Cluster`.
        :returns: List of `otcextensions.sdk.dws.v1.tag.Tag` instances.
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._list(_tag.Tag, cluster_id=cluster.id)

    def create_cluster_tag(self, cluster, tag):
        """
        Create a new tag for a DWS cluster.

        :param cluster: Cluster ID or an instance of
            `otcextensions.sdk.dws.v1.cluster.Cluster`.
        :param tag: Dictionary with 'key' and 'value' for the tag.
        :returns: Created tag instance.
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._create(_tag.Tag, cluster_id=cluster.id, **tag)

    def delete_cluster_tag(self, cluster, tag_key, ignore_missing=True):
        """
        Delete a single tag from a DWS cluster.

        :param cluster: Cluster ID or an instance of
            `otcextensions.sdk.dws.v1.cluster.Cluster`.
        :param tag_key: The key of the tag to be deleted.
        :param ignore_missing: When False,
            `openstack.exceptions.ResourceNotFound`
            will be raised when the tag does not exist.
            When True, no exception will be set when attempting
            to delete a nonexistent tag.
        :returns: None
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        tag_obj = _tag.Tag.existing(key=tag_key, cluster_id=cluster.id)
        return self._delete(
            _tag.Tag,
            tag_obj.key,
            cluster_id=cluster.id,
            ignore_missing=ignore_missing,
        )

    def batch_create_cluster_tags(self, cluster, tags):
        """
        Batch addition of tags for a cluster.

        :param cluster: Cluster ID or an instance of
            `otcextensions.sdk.dws.v1.cluster.Cluster`.
        :param tags: List of dictionaries of tags to add.
        :return: A list of created
            `otcextensions.sdk.dws.v1.tag.Tag` instances.
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return _tag.Tag().manage_tags_batch(self, cluster.id, tags, 'create')

    def batch_delete_cluster_tags(self, cluster, tags):
        """
        Batch deletion of tags from a cluster.

        :param cluster: Cluster ID or an instance of
            `otcextensions.sdk.dws.v1.cluster.Cluster`.
        :param tags: List of dictionaries of tags to delete.
        :return: A list of deleted
            `otcextensions.sdk.dws.v1.tag.Tag` instances.
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return _tag.Tag().manage_tags_batch(self, cluster.id, tags, 'delete')

    def wait_for_delete(self, cluster, interval=20, wait=12000, callback=None):
        """Wait for a resource to be deleted.

        :param cluster: The resource to wait on to be deleted.
        :param interval: Number of seconds to wait before to consecutive
            checks. Default to 2.
        :param wait: Maximum number of seconds to wait before the change.
            Default to 120.
        :param callback: A callback function. This will be called with a single
            value, progress, which is a percentage value from 0-100.

        :returns: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
            to delete failed to occur in the specified seconds.
        """
        return resource.wait_for_delete(self,
                                        cluster,
                                        interval,
                                        wait,
                                        callback)

    def _get_cleanup_dependencies(self):
        return {
            'dws': {
                'before': ['network']
            }
        }

    def _service_cleanup(
        self,
        dry_run=True,
        client_status_queue=None,
        identified_resources=None,
        filters=None,
        resource_evaluation_fn=None,
        skip_resources=None,
    ):
        if self.should_skip_resource_cleanup("cluster", skip_resources):
            return

        clusters = []
        for cluster in self.clusters():
            if not dry_run and cluster.status == 'AVAILABLE':
                tags = list(self.cluster_tags(cluster))
                for tag in tags:
                    self._service_cleanup_del_res(
                        self.delete_cluster_tag,
                        tag['key'],
                        dry_run=dry_run,
                        client_status_queue=client_status_queue,
                        identified_resources=identified_resources,
                        filters=filters,
                        resource_evaluation_fn=resource_evaluation_fn,
                    )
            need_delete = self._service_cleanup_del_res(
                self.delete_cluster,
                cluster,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn,
            )
            if not dry_run and need_delete:
                clusters.append(cluster)

        for cluster in clusters:
            self.wait_for_delete(cluster)

        for snapshot in self.snapshots():
            self._service_cleanup_del_res(
                self.delete_snapshot,
                snapshot,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn,
            )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.waf.v1 import _base


class ServerEntry(resource.Resource):
    #: Properties
    #: IP address
    address = resource.Body('address')
    #: Client protocol
    client_protocol = resource.Body('client_protocol')
    #: Server protocol
    server_protocol = resource.Body('server_protocol')
    #: Port number
    port = resource.Body('port', type=int)


class Domain(_base.Resource):
    """WAF Domain Resource"""
    resources_key = 'items'
    base_path = '/waf/instance'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'name', 'hostname', 'policy_name', 'limit', 'offset',
        policy_name='policyname',
        name='hostname')

    #: Properties
    #: Specifies whether a domain name is connected to WAF
    access_status = resource.Body('access_status', type=int)
    #: Certificate ID.
    #: This parameter is mandatory when client_protocol is set to HTTPS.
    certificate_id = resource.Body('certificate_id')
    #: CNAME
    cname = resource.Body('cname')
    #: domain name
    name = resource.Body('hostname', aka='hostname')
    #: Specifies the policy ID.
    policy_id = resource.Body('policy_id')
    #: protocol type of the client.
    #: The options are HTTP, HTTPS, and HTTP,HTTPS.
    protocol = resource.Body('protocol')
    #: WAF mode.
    protect_status = resource.Body('protect_status', type=int)
    #: Specifies whether a proxy is configured.
    proxy = resource.Body('proxy', type=bool)
    #: Specifies the origin server information, including the client_protocol,
    #: server_protocol, address, and port fields.
    server = resource.Body('server', type=list, list_type=ServerEntry)
    #: source IP header. This parameter is required only when proxy is set
    #: to true.
    #: The options are as follows: default, cloudflare, akamai, and custom.
    sip_header_name = resource.Body('sip_header_name')
    #: Specifies the HTTP request header for identifying the real source IP
    #: address. This parameter is required only when proxy is set to true.
    #: - If sip_header_name is default, sip_header_list is ["X-Forwarded-For"].
    #: - If sip_header_name is cloudflare, sip_header_list is
    #: ["CF-Connecting-IP", "X-Forwarded-For"].
    #: - If sip_header_name is akamai, sip_header_list is ["True-Client-IP"].
    #: - If sip_header_name is custom, you can customize a value.
    sip_header_list = resource.Body('sip_header_list', type=list)
    #: subdomain name.
    #: This parameter is returned only when proxy is set to true.
    subdomain = resource.Body('sub_domain')
    #: Certificate uploading timestamp
    timestamp = resource.Body('timestamp')
    #: TXT record. This parameter is returned only when proxy is set to true.
    txt_record = resource.Body('txt_code')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from openstack import resource
from otcextensions.sdk.dns.v2 import nameserver as _ns
from otcextensions.sdk.dns.v2 import recordset as _rs
from otcextensions.sdk.dns.v2 import zone as _zone
from otcextensions.sdk.dns.v2 import floating_ip as _fip


class Proxy(proxy.Proxy):

    # ======== Zones ========
    def zones(self, **query):
        """Retrieve a generator of zones

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `zone_type`: Zone Type
            * `marker`:  pagination marker
            * `limit`: pagination limit
            * `offset`: pagination offset

        :returns: A generator of zone
            :class:`~otcextensions.sdk.dns.v2.zone.Zone` instances
        """
        return self._list(_zone.Zone, **query)

    def create_zone(self, **attrs):
        """Create a new zone from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~otcextensions.sdk.dns.v2.zone.Zone`,
                           comprised of the properties on the Zone class.
        :returns: The results of zone creation
        :rtype: :class:`~otcextensions.sdk.dns.v2.zone.Zone`
        """
        return self._create(_zone.Zone, prepend_key=False, **attrs)

    def get_zone(self, zone):
        """Get a zone

        :param zone: The value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :returns: Zone instance
        :rtype: :class:`~otcextensions.sdk.dns.v2.zone.Zone`
        """
        return self._get(_zone.Zone, zone)

    def delete_zone(self, zone, ignore_missing=True):
        """Delete a zone

        :param zone: The value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the zone does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent zone.

        :returns: Zone been deleted
        :rtype: :class:`~otcextensions.sdk.dns.v2.zone.Zone`
        """
        return self._delete(_zone.Zone, zone, ignore_missing=ignore_missing)

    def update_zone(self, zone, **attrs):
        """Update zone attributes

        :param zone: The id or an instance of
            :class:`~otcextensions.sdk.dns.v2.zone.Zone`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.dns.v2.zone.Zone`

        :rtype: :class:`~otcextensions.sdk.dns.v2.zone.Zone`
        """
        return self._update(_zone.Zone, zone, **attrs)

    def find_zone(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single zone

        :param name_or_id: The name or ID of a zone
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the zone does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent zone.

        :returns: ``None``
        """
        return self._find(_zone.Zone, name_or_id,
                          ignore_missing=ignore_missing,
                          **attrs)

    def add_router_to_zone(self, zone, **router):
        """Add router(VPC) to private zone

        :param zone: The value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :param router: The parameter router_id is mandatory
        :returns: updater instance of
            :class:`~otcextensions.sdk.dns.v2.zone.Router`
        :rtype: :class:`~otcextensions.sdk.dns.v2.zone.Router`
        """
        zone = self._get_resource(_zone.Zone, zone)
        return zone.associate_router(self, **router)

    def remove_router_from_zone(self, zone, **router):
        """Remove router(VPC) from private zone

        :param zone: The value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :param router: The parameter router_id is mandatory
        :returns: updater instance of
            :class:`~otcextensions.sdk.dns.v2.zone.Router`
        :rtype: :class:`~otcextensions.sdk.dns.v2.zone.Router`
        """
        zone = self._get_resource(_zone.Zone, zone)
        return zone.disassociate_router(self, **router)

    def wait_for_zone(self, zone, status='ACTIVE', failures=None,
                      interval=2, wait=180, attribute='status'):
        """Wait for an zone to be in a particular status.
        :param zone:
            The :class:`~otcextensions.sdk.dns.v2.zone.Zone`
            or zone ID to wait on to reach the specified status.
        :param status: Desired status.
        :param failures:
            Statuses that would be interpreted as failures.
        :type failures: :py:class:`list`
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        failures = ['ERROR'] if failures is None else failures
        return resource.wait_for_status(
            self, zone, status, failures, interval, wait)

    def wait_for_delete_zone(self, zone, interval=2, wait=180):
        """Wait for the zone to be deleted.
        :param zone:
            The :class:`~otcextensions.sdk.dns.v2.zone.Zone`
            or zone ID to wait on to be deleted.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait for the delete.
            Default to 180.
        :return: Method returns self on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        """
        zone = self._get_resource(_zone.Zone, zone)
        return resource.wait_for_delete(self, zone, interval, wait)

    # ======== Nameservers ========
    def nameservers(self, zone):
        """list nameservers of Zone

        :param zone: The value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :returns: list of `otcextensions.sdk.dns.v2.nameserver.NameServer`
            instance
        :rtype: :class:`~otcextensions.sdk.dns.v2.nameserver.NameServer`
        """
        instance = self._get_resource(_zone.Zone, zone)
        return self._list(_ns.NameServer, paginated=False,
                          zone_id=instance.id)

    # ======== Recordsets ========
    def recordsets(self, zone=None, **query):
        """Retrieve a generator of recordsets which belongs to `zone`

        :param zone: The optional value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `marker`:  pagination marker
            * `limit`: pagination limit

        :returns: A generator of zone
            (:class:`~otcextensions.sdk.dns.v2.recordset.Recordset`) instances
        """
        base_path = None
        if not zone:
            base_path = '/recordsets'
        else:
            zone = self._get_resource(_zone.Zone, zone)
            query.update({'zone_id': zone.id})
        return self._list(_rs.Recordset, base_path=base_path, **query)

    def create_recordset(self, zone, **attrs):
        """Create a new recordset in the zone

        :param zone: The value can be the ID of a zone
            or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`,
            comprised of the properties on the Recordset class.
        :returns: The results of zone creation
        :rtype: :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`
        """
        zone = self._get_resource(_zone.Zone, zone)
        attrs.update({'zone_id': zone.id})
        return self._create(_rs.Recordset, prepend_key=False, **attrs)

    def update_recordset(self, recordset, **attrs):
        """Update Recordset attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`,
            comprised of the properties on the Recordset class.
        :returns: The results of zone creation
        :rtype: :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`
        """
        return self._update(_rs.Recordset, recordset, **attrs)

    def get_recordset(self, recordset, zone):
        """Get a recordset

        :param zone: The value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :returns: Recordset instance
        :rtype: :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`
        """
        zone = self._get_resource(_zone.Zone, zone)
        return self._get(_rs.Recordset, recordset, zone_id=zone.id)

    def delete_recordset(self, recordset, zone=None, ignore_missing=True):
        """Delete a zone

        :param recordset: The value can be the ID of a recordset
             or a :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`
             instance.
        :param zone: The value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the zone does not exist. When set to ``True``, no exception will
            be set when attempting to delete a nonexistent zone.

        :returns: Recordset instance been deleted
        :rtype: :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`
        """
        if zone:
            zone = self._get_resource(_zone.Zone, zone)
            recordset = self._get(
                _rs.Recordset, recordset, zone_id=zone.id)
        return self._delete(_rs.Recordset, recordset,
                            ignore_missing=ignore_missing)

    def find_recordset(self, zone, name_or_id, ignore_missing=True, **attrs):
        """Find a single recordset

        :param zone: The value can be the ID of a zone
             or a :class:`~otcextensions.sdk.dns.v2.zone.Zone` instance.
        :param name_or_id: The name or ID of a recordset
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the recordset does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent recordset.

        :returns: :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`
        """
        zone = self._get_resource(_zone.Zone, zone)
        return self._find(_rs.Recordset, name_or_id,
                          ignore_missing=ignore_missing, zone_id=zone.id,
                          **attrs)

    def wait_for_recordset(self, recordset, status='ACTIVE', failures=None,
                           interval=2, wait=180, attribute='status'):
        """Wait for an recordset to be in a particular status.
        :param zone:
            The :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`
            or recordset ID to wait on to reach the specified status.
        :param status: Desired status.
        :param failures:
            Statuses that would be interpreted as failures.
        :type failures: :py:class:`list`
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait before the change.
            Default to 180
        :return: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
                 to the desired status failed to occur in specified seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """
        failures = ['ERROR'] if failures is None else failures
        return resource.wait_for_status(
            self, recordset, status, failures, interval, wait)

    def wait_for_delete_recordset(self, recordset, interval=2, wait=180):
        """Wait for the recordset to be deleted.
        :param recordset:
            The :class:`~otcextensions.sdk.dns.v2.recordset.Recordset`
            or recordset ID to wait on to be deleted.
        :param int interval:
            Number of seconds to wait before to consecutive checks.
            Default to 2.
        :param int wait:
            Maximum number of seconds to wait for the delete.
            Default to 180.
        :return: Method returns self on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
                 to status failed to occur in wait seconds.
        """
        recordset = self._get_resource(_rs.Recordset, recordset)
        return resource.wait_for_delete(self, recordset, interval, wait)

    # ======== FloatingIPs ========
    def floating_ips(self, **query):
        """Retrieve a generator of FloatingIP PTR records

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `name`: Recordset Name field.
            * `type`: Type field.
            * `status`: Status of the recordset.
            * `ttl`: TTL field filter.
            * `description`: Recordset description field filter.

        :returns: A generator of floatingips
            (:class:`~otcextensions.sdk.dns.v2.floating_ip.FloatingIP`)
            instances
        """
        return self._list(_fip.FloatingIP, **query)

    def get_floating_ip(self, floating_ip):
        """Get a Floating IP

        :param floating_ip: The value can be the ID of a floating ip
             or a :class:`~otcextensions.sdk.dns.v2.floating_ip.FloatingIP`
             instance.
             The ID is in format "region_name:floatingip_id"
        :returns: FloatingIP instance.
        :rtype: :class:`~otcextensions.sdk.dns.v2.floating_ip.FloatingIP`
        """
        return self._get(_fip.FloatingIP, floating_ip)

    def set_floating_ip(self, floating_ip, **attrs):
        """Set a Floating IP PTR record

        :param floating_ip: the Floating IP ID

        :returns: FloatingIP PTR record.
        :rtype: :class:`~otcextensions.sdk.dns.v2.floating_ip.FloatingIP`
        """
        return self._update(_fip.FloatingIP, floating_ip, **attrs)

    def update_floating_ip(self, floating_ip, **attrs):
        """Update floating ip attributes

        :param floating_ip: The id or an instance of
            :class:`~otcextensions.sdk.dns.v2.fip.FloatingIP`.
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.dns.v2.fip.FloatingIP`.

        :rtype: :class:`~otcextensions.sdk.dns.v2.fip.FloatingIP`
        """
        return self._update(_fip.FloatingIP, floating_ip, **attrs)

    def unset_floating_ip(self, floating_ip):
        """Unset a Floating IP PTR record

        :param floating_ip: ID for the floatingip associated with the
            project.

        :returns: FloatingIP PTR record.
        :rtype: :class:`~otcextensions.sdk.dns.v2.floating_ip.FloatipgIP`
        """
        # concat `region:floating_ip_id` as id
        attrs = {'ptrdname': None}
        return self._update(_fip.FloatingIP, floating_ip, **attrs)

    def _get_cleanup_dependencies(self):
        # DNS may depend on floating ip
        return {
            'dns': {
                'before': ['network']
            }
        }

    def _service_cleanup(self, dry_run=True, client_status_queue=False,
                         identified_resources=None,
                         filters=None, resource_evaluation_fn=None,
                         skip_resources=None):
        # Delete all public zones
        for obj in self.zones():
            self._service_cleanup_del_res(
                self.delete_zone,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)
        # Delete all private zones
        for obj in self.zones(type='private'):
            self._service_cleanup_del_res(
                self.delete_zone,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)
        # Unset all floatingIPs
        # NOTE: FloatingIPs are not cleaned when filters are set
        for obj in self.floating_ips():
            self._service_cleanup_del_res(
                self.unset_floating_ip,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts labels v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListDatasetLabels(command.Lister):
    _description = _("This API is used to query all samples of a dataset.")
    columns = ("name", "type")

    def get_parser(self, prog_name):
        parser = super(ListDatasetLabels, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        parser.add_argument(
            "--version-id",
            metavar="<version_id>",
            help=_("Dataset version id."),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        params = {}
        if parsed_args.version_id:
            params["version_id"] = parsed_args.version_id
        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        data = client.dataset_labels(dataset.id, **params)

        formatters = {}
        return (
            self.columns,
            (
                utils.get_dict_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.css.v1 import cluster_upgrade_status
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = {
    'id': '0249620d-1c4a-4211-943a-ced7b9a3cda5',
    'startTime': '2024-07-19T08:58:52',
    'endTime': '2024-07-19T09:16:20',
    'status': 'FAILED',
    'agencyName': 'css_upgrade_agency',
    'imageInfo': {
        'id': '5670dd9f-6399-4a2b-9faf-2a24c2a07cd3',
        'displayName': '7.10.2_23.5.1_0123',
        'imageDesc': 'The latest image of...',
        'datastoreType': 'elasticsearch',
        'datastoreVersion': '7.10.2',
        'priority': 11,
    },
    'totalNodes': 'csstest0716-ess-esn-2-1...',
    'completedNodes': '',
    'currentNodeName': 'csstest0716-ess-esn-2-1',
    'executeTimes': '1',
    'currentNodeDetail': [
        {
            'order': 0,
            'name': 'Data migration',
            'status': 'SUCCESS',
            'desc': 'Data is migrated.',
            'beginTime': '2024-07-19T08:58:54',
            'endTime': '2024-07-19T08:58:59',
        },
        {
            'order': 1,
            'name': 'Node brought offline',
            'status': 'SUCCESS',
            'desc': 'The node is brought offline.',
            'beginTime': '2024-07-19T08:59:04',
            'endTime': '2024-07-19T09:12:55',
        },
        {
            'order': 2,
            'name': 'Node creation',
            'status': 'SUCCESS',
            'desc': 'The node is rebuilt with the same specifications.',
            'beginTime': '2024-07-19T09:13:00',
            'endTime': '2024-07-19T09:14:43',
        },
        {
            'order': 3,
            'name': 'Connectivity test',
            'status': 'SUCCESS',
            'desc': 'Node creation is verified.',
            'beginTime': '2024-07-19T09:14:47',
            'endTime': '2024-07-19T09:15:19',
        },
    ],
    'failMessage': '[402c6c3]RdsBindExistPortToInstanceTask-fail:null',
}


class TestClusterUpgradeStatus(base.TestCase):
    def setUp(self):
        super(TestClusterUpgradeStatus, self).setUp()

    def test_basic(self):
        sot = cluster_upgrade_status.ClusterUpgradeStatus()

        self.assertEqual(
            '/clusters/%(cluster_id)s/upgrade/detail', sot.base_path
        )
        self.assertEqual(None, sot.resource_key)
        self.assertEqual('detailList', sot.resources_key)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        updated_sot_attrs = {
            'startTime': 'start_time',
            'endTime': 'end_time',
            'agencyName': 'agency_name',
            'totalNodes': 'total_nodes',
            'completedNodes': 'completed_nodes',
            'currentNodeName': 'current_node_name',
            'executeTimes': 'execute_times',
            'failMessage': 'fail_message',
        }
        sot = cluster_upgrade_status.ClusterUpgradeStatus(**EXAMPLE)

        for key, value in EXAMPLE.items():
            if key in updated_sot_attrs.keys():
                self.assertEqual(
                    getattr(sot, updated_sot_attrs[key]), EXAMPLE[key]
                )
            elif key == 'imageInfo':
                pass
            elif key == 'currentNodeDetail':
                pass
            else:
                assert_attributes_equal(self, getattr(sot, key), value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base
from otcextensions.sdk import ak_auth

from otcextensions.sdk.obs.v1 import container


EXAMPLE = {
    "Name": "container_name",
    "CreationDate": "2013-01-15T05:52:15.920Z<",
}

EXAMPLE_LIST = """<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<ListAllMyBucketsResult xmlns="http://obs.otc.t-systems.com/doc/2016-01-01/">
<Owner><ID>some_id</ID>
<DisplayName></DisplayName>
</Owner><Buckets>
<Bucket><Name>test-v1</Name>
<CreationDate>2018-10-02T08:54:55.885Z</CreationDate></Bucket>
<Bucket><Name>test-v2</Name>
<CreationDate>2018-09-19T12:47:22.205Z</CreationDate></Bucket>
</Buckets></ListAllMyBucketsResult>
"""


class TestContainer(base.TestCase):

    def setUp(self):
        super(TestContainer, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = container.Container()

        self.assertEqual('/', sot.base_path)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_head)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):

        sot = container.Container(**EXAMPLE)
        self.assertEqual(EXAMPLE['Name'], sot.id)
        self.assertEqual(EXAMPLE['Name'], sot.name)
        self.assertEqual(EXAMPLE['CreationDate'], sot.creation_date)

    def test_list(self):

        sot = container.Container()

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.content = EXAMPLE_LIST

        self.sess.get.return_value = mock_response

        result = list(sot.list(
            self.sess,
        ))

        self.assertEqual(2, len(result))
        self.assertEqual('test-v1', result[0].name)
        self.assertEqual('test-v2', result[1].name)

    def test_create(self):

        sot = container.Container(name='test-v1')

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.content = ''
        auth = ak_auth.AKRequestsAuth(
            access_key='test-ak',
            secret_access_key='test-sk',
            host='http://test-sk',
            region='',
            service='s3'
        )
        self.sess.put.return_value = mock_response

        sot.create(
            self.sess,
            endpoint_override='epo',
            requests_auth=auth)

        self.sess.put.assert_called_once_with(
            '/',
            data=None,
            endpoint_override='epo',
            requests_auth=auth
        )
